Package: arg
Type: Package
Title: Clean and Simple Argument Checking
Version: 0.2.1
Authors@R: person("Noah", "Greifer", role = c("aut", "cre"),
         email = "noah.greifer@gmail.com",
	       comment = c(ORCID="0000-0003-3067-7154"))
Description: Checks function arguments, ideally for use in R packages. Uses a simple interface and produces clean, informative error messages using 'cli'.
Depends: R (>= 4.1.0)
Imports: rlang (>= 1.2.0), cli (>= 3.6.4), stats, utils
License: GPL (>= 2)
Encoding: UTF-8
URL: https://ngreifer.github.io/arg/, https://github.com/ngreifer/arg
BugReports: https://github.com/ngreifer/arg/issues
Config/roxygen2/version: 8.0.0
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-29 16:46:22 UTC; NoahGreifer
Author: Noah Greifer [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3067-7154>)
Maintainer: Noah Greifer <noah.greifer@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-30 05:10:13 UTC
Built: R 4.5.2; ; 2026-07-30 06:30:45 UTC; unix

# Generated by roxygen2: do not edit by hand

export(arg_all_NA)
export(arg_and)
export(arg_array)
export(arg_atomic)
export(arg_between)
export(arg_character)
export(arg_colnamed)
export(arg_color)
export(arg_colors)
export(arg_colour)
export(arg_colours)
export(arg_cor)
export(arg_count)
export(arg_counts)
export(arg_cov)
export(arg_data)
export(arg_data.frame)
export(arg_distance)
export(arg_dots_not_supplied)
export(arg_dots_supplied)
export(arg_element)
export(arg_env)
export(arg_equal)
export(arg_factor)
export(arg_flag)
export(arg_formula)
export(arg_function)
export(arg_gt)
export(arg_gte)
export(arg_index)
export(arg_indices)
export(arg_is)
export(arg_is_NA)
export(arg_is_not)
export(arg_length)
export(arg_list)
export(arg_logical)
export(arg_lt)
export(arg_lte)
export(arg_matrix)
export(arg_name)
export(arg_named)
export(arg_names)
export(arg_no_NA)
export(arg_non_null)
export(arg_not_element)
export(arg_not_equal)
export(arg_null)
export(arg_number)
export(arg_numeric)
export(arg_or)
export(arg_ordered)
export(arg_string)
export(arg_supplied)
export(arg_symmetric)
export(arg_unique)
export(arg_vector)
export(arg_whole_number)
export(arg_whole_numeric)
export(err)
export(match_arg)
export(msg)
export(when_not_null)
export(when_supplied)
export(wrn)
importFrom(stats,setNames)
importFrom(utils,capture.output)

# arg

[![Actions Status](https://github.com/DoumanAsh/arg.rs/workflows/Rust/badge.svg)](https://github.com/DoumanAsh/arg.rs/actions)
[![Crates.io](https://img.shields.io/crates/v/arg.svg)](https://crates.io/crates/arg)
[![Documentation](https://docs.rs/arg/badge.svg)](https://docs.rs/crate/arg/)

Very simple code generator for command line argument parsing.

No dependencies except proc macro (with its dependencies) used at build time.

## Features

- `std` - Enables utilities that require `std` library.

## Debug

Set env variable `ARG_RS_PRINT_PARSER` to any value except `0` or `false` to print parser result

<!-- README.md is generated from README.Rmd. Please edit that file -->

# *arg*: Clean and Simple Argument Checking

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/arg)](https://CRAN.R-project.org/package=arg)
<!-- badges: end -->

## Overview

*arg* produces nice, clean error messages for checking function
arguments in R packages. I developed *arg* because I often found myself
writing the same error messages in my packages, and I wasn’t satisfied
with the other similar R packages available. *arg* uses
[*cli*](https://cli.r-lib.org/) to produce clean and clear errors
without requiring much programming from the developer. These messages
are designed to be clear to the user, avoiding complicated language.
Many messages adapt to the type of (incorrect) input received.

*arg* contains functions for common arguments checks, such as checking
that an argument is a single number (`arg_number()`), a string
(`arg_string()`), a single `TRUE`/`FALSE` value (`arg_flag()`), or a
data frame or matrix (`arg_data()`). In addition, it is possible to
build more complex argument checks using `arg_and()` and `arg_or()`,
which require that all or at least one check is passed, and
`when_not_null()` and `when_supplied()`, which allow for `NULL` or
omitted arguments, respectively.

``` r
library(arg)

z <- "a string"

# Check that `z` is a string
arg_string(z)

# Check that `z` is a number
arg_number(z)
#> Error:
#> ! `z` must be a number.

# Check that `z` is a string of length 2
arg_and(z,
        arg_string,
        arg_length(2L))
#> Error:
#> ! All of the following conditions must be met:
#> ✔ `z` must be a string
#> ✖ `z` must have length 2

# Check that `z` is NA, a flag, or a count
arg_or(z,
       arg_is_NA,
       arg_flag,
       arg_count)
#> Error:
#> ! `z` must be NA, a logical value (TRUE or FALSE), or a count (a
#>   non-negative whole number).
```

*arg* is meant to be used inside other R packages. Its error-throwing
function `err()` (a wrapper for `rlang::abort()`) automatically includes
the relevant (user-facing) function in its error message.

The name “arg” is meant to be short for “argument”, but also perhaps
represents the sound you would make if you were to encounter an error
when running some code. Hopefully *arg* reduces the number of times
users say “arg!”.

<!-- If you use any of my R packages, including [*MatchIt*], [*WeightIt*], [*cobalt*], [*optweight*], or [*adrftools*], you may already be using *arg*!
-->

## Installation

You can install the development version of *arg* from
[GitHub](https://github.com/ngreifer/arg) with:

``` r
# install.packages("pak")
pak::pak("ngreifer/arg")
```

You can install the stable version on CRAN using

``` r
install.packages("arg")
```

## Related packages

*arg* serves a similar function to other existing packages. These
packages may be better for your purposes, and *arg* is not meant to
compete with them.

- [*assertions*](https://selkamand.github.io/assertions/)
- [*checkmate*](https://mllg.github.io/checkmate/)
- [*checkarg*](https://cran.r-project.org/package=checkarg)
- [*chk*](https://poissonconsulting.github.io/chk/)
- [*dreamerr*](https://cran.r-project.org/package=dreamerr)
- [*erify*](https://flujoo.github.io/erify/)
- [*favr*](https://lj-jenkins.github.io/favr/)

#' Check Supplied Argument
#'
#' Checks whether an argument with no default value was supplied. An error will be thrown if `rlang::is_missing(x)` is `TRUE`, i.e., if an argument with no default is omitted from a function call.
#'
#' @inheritParams arg_is
#'
#' @inherit arg_is return
#'
#' @seealso [arg_non_null()], [arg_no_NA()], [rlang::is_missing()], [rlang::check_required()]
#'
#' @examples
#' f <- function(z) {
#'   arg_supplied(z)
#' }
#'
#' try(f(1)) ## No error: argument supplied
#' try(f())  ## Error!
#'
#' # Will not throw for NULL or default arguments
#' try(f(NULL)) ## No error: argument supplied
#'
#' f2 <- function(z = NULL) {
#'   arg_supplied(z)
#' }
#'
#' try(f2()) ## No error; default provided

#' @export
arg_supplied <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                         .call) {
  if (rlang::is_missing(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    arg_expr <- substitute(x)

    if (!rlang::is_symbol(arg_expr)) {
      err("{.arg x} must be an argument name",
          .call = rlang::current_env()) |>
        internal_arg()
    }

    err("an argument to {.arg {(.arg)}} must be supplied",
        .call = .call)
  }
}

#' Check Symmetric Matrix Argument
#'
#' `arg_symmetric()` checks whether an argument is a square, symmetric, numeric matrix. `arg_cov()` checks whether an argument is like a covariance matrix (i.e., square and symmetric with non-negative diagonal entries). `arg_cor()` checks whether an argument is like a correlation matrix (i.e., square and symmetric with all values between -1 and 1 and ones on the diagonal). `arg_distance()` checks whether an argument is like a distance matrix (i.e., square and symmetric with non-negative entries and zeros on the diagonal).
#'
#' @inheritParams arg_is
#' @param tol `numeric`; the tolerance value used to assess symmetry and any numerical bounds. Passed to [isSymmetric()].
#' @param \dots other arguments passed to [isSymmetric()] (and eventually to [all.equal()]).
#'
#' @details
#' These functions check that an argument is a square, symmetric, numeric matrix. This can be useful when a function can accept a covariance matrix, correlation matrix, or distance matrix. `arg_cov()` will throw an error if any values on its diagonal are less than `-tol`. `arg_cor()` will throw an error if any of its values are greater than `1 + tol` in absolute value or if the diagonal entries are not between `-1 - tol` and `1 + tol`. `arg_distance()` will thrown an error if any of its values are less than `-tol` or if the diagonal entries are not between `-tol` and `tol`. The tolerance is just slightly greater than 0 to allow for numeric imprecision.
#'
#' No checks on semi-definiteness or rank are performed.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_numeric()], [arg_matrix()], [arg_between()], [isSymmetric()]
#'
#' @examples
#' set.seed(1234)
#' mat <- matrix(rnorm(12), ncol = 3L) # Non square
#' sym_mat <- -crossprod(mat)          # Square, symmetric
#' cov_mat <- cov(mat)                 # Covariance
#' cor_mat <- cor(mat)                 # Correlation
#' dist_mat <- as.matrix(dist(mat))    # Distance
#'
#' try(arg_symmetric(mat))     # Error: not square
#' try(arg_symmetric(sym_mat)) # No error
#'
#' try(arg_cov(sym_mat)) # Error: diagonal must be non-negative
#' try(arg_cov(cov_mat)) # No error
#'
#' try(arg_cor(cov_mat)) # Error: values must be in [-1, 1]
#' try(arg_cor(cor_mat)) # No error
#'
#' try(arg_distance(cor_mat))  # Error: diagonal must be 0
#' try(arg_distance(dist_mat)) # No error

#' @export
arg_symmetric <- function(x, tol = 100 * .Machine$double.eps, ...,
                          .arg = rlang::caller_arg(x), .msg = NULL,
                          .call) {
  arg_number(tol, .call = rlang::current_env()) |>
    internal_arg()
  arg_gte(tol, 0, .call = rlang::current_env()) |>
    internal_arg()

  is_symmetric_mat <- is.matrix(x) && nrow(x) == ncol(x) &&
    is.numeric(x) && isSymmetric(unname(x), tol = tol, ...)

  if (!is_symmetric_mat) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (isTRUE(all.equal(tol, Inf))) {
      err("{.arg {(.arg)}} must be a square, numeric matrix",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a square, symmetric, numeric matrix",
        .call = .call)
  }
}

#' @export
#' @rdname arg_symmetric
arg_cov <- function(x, tol = 100 * .Machine$double.eps, ...,
                    .arg = rlang::caller_arg(x), .msg = NULL,
                    .call) {
  arg_number(tol, .call = rlang::current_env()) |>
    internal_arg()
  arg_gte(tol, 0, .call = rlang::current_env()) |>
    internal_arg()

  is_symmetric_mat <- is.matrix(x) && nrow(x) == ncol(x) &&
    is.numeric(x) && isSymmetric(unname(x), tol = tol, ...)

  if (!is_symmetric_mat ||
      any(diag(x) < -tol)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_symmetric_mat) {
      err("{.arg {(.arg)}} must be a square, symmetric, numeric matrix",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a square, symmetric matrix with non-negative values on the diagonal",
        .call = .call)
  }
}

#' @export
#' @rdname arg_symmetric
arg_cor <- function(x, tol = 100 * .Machine$double.eps, ...,
                    .arg = rlang::caller_arg(x), .msg = NULL,
                    .call) {

  arg_number(tol, .call = rlang::current_env()) |>
    internal_arg()
  arg_gte(tol, 0, .call = rlang::current_env()) |>
    internal_arg()

  is_symmetric_mat <- is.matrix(x) && nrow(x) == ncol(x) &&
    is.numeric(x) && isSymmetric(unname(x), tol = tol, ...)

  if (!is_symmetric_mat ||
      any(abs(x) > 1 + tol) ||
      any(abs(diag(x) - 1) > tol)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_symmetric_mat) {
      err("{.arg {(.arg)}} must be a square, symmetric, numeric matrix",
          .call = .call)
    }

    if (any(abs(x) > 1 + tol)) {
      err("all values in {.arg {(.arg)}} must be between {.val {c(-1, 1)}}",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a square, symmetric matrix with ones on the diagonal",
        .call = .call)
  }
}

#' @export
#' @rdname arg_symmetric
arg_distance <- function(x, tol = 100 * .Machine$double.eps, ...,
                         .arg = rlang::caller_arg(x), .msg = NULL,
                         .call) {
  arg_number(tol, .call = rlang::current_env()) |>
    internal_arg()
  arg_gte(tol, 0, .call = rlang::current_env()) |>
    internal_arg()

  is_symmetric_mat <- is.matrix(x) && nrow(x) == ncol(x) &&
    is.numeric(x) && isSymmetric(unname(x), tol = tol, ...)

  if (!is_symmetric_mat ||
      any(abs(diag(x)) > tol) ||
      any(x < -tol)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_symmetric_mat) {
      err("{.arg {(.arg)}} must be a square, symmetric, numeric matrix",
          .call = .call)
    }

    if (any(abs(diag(x)) > tol)) {
      err("{.arg {(.arg)}} must be a square, symmetric matrix with zeros on the diagonal",
          .call = .call)
    }

    err("all values in {.arg {(.arg)}} must be non-negative",
        .call = .call)
  }
}

#' Check Character Argument
#'
#' @description
#' Checks whether an argument is a character vector (`arg_character()`), a character scalar (`arg_string()`), or a factor (`arg_factor()`).
#'
#' @inheritParams arg_is
#'
#' @details
#' `NA` values in `arg_string()` will cause an error to be thrown.
#'
#' @inherit arg_is return
#'
#' @seealso [is.character()], [is.factor()], [rlang::is_string()]
#'
#' @examples
#' f <- function(z) {
#'   arg_string(z)
#' }
#'
#' try(f("a")) # No error
#' try(f(c("a", "b"))) # Error: arg_string() requires scalar
#' try(f(NA)) # NAs not allowed for arg_string()

#' @export
arg_character <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                          .call) {
  if (!is.character(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a character vector",
        .call = .call)
  }
}

#' @export
#' @rdname arg_character
arg_string <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is_scalar(x) || !is.character(x) || anyNA(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a string",
        .call = .call)
  }
}

#' @export
#' @rdname arg_character
arg_factor <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is.factor(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a factor",
        .call = .call)
  }
}

#' Check Element
#'
#' Checks whether values in an argument are all elements of some set (`arg_element()`) or are not all elements of a set (`arg_not_element()`). `arg_element()` throws an error when `all(is.element(x, values))` is `FALSE`, and `arg_not_element()` throws an error when `any(is.element(x, values))` is `TRUE`.
#'
#' @inheritParams arg_is
#' @param values the values to check `x` against.
#'
#' @details
#' `arg_element()` can be useful for checking whether an argument matches one or more allowed values. It's important that a check is done beforehand to ensure `x` is the correct type, e.g., using [arg_string()] or [arg_character()] if `values` is a character vector. No partial matching is used; the values must match exactly. Use [match_arg()] to allow partial matching and return the full version of the supplied argument.
#'
#' `arg_not_element()` can be useful for ensuring there is no duplication of values between an argument and some other set.
#'
#' @inherit arg_is return
#'
#' @seealso [is.element()], [match_arg()]
#'
#' @examples
#' f <- function(z) {
#'   arg_element(z, c("opt1", "opt2", "opt3"))
#' }
#'
#' try(f("opt1"))            # No error
#' try(f(c("opt1", "opt2"))) # No error, all are elements
#' try(f(c("opt1", "opt1"))) # No error: repeats allowed
#' try(f("bad_arg"))         # Error: not an element of set
#' try(f("opt"))             # Error: partial matching not allowed
#' try(f(c("opt1", "bad_arg"))) # Error: one non-match
#'
#' g <- function(z) {
#'   arg_not_element(z, c("bad1", "bad2", "bad3"))
#' }
#'
#' try(g("bad1"))            # Error: z is an element
#' try(g(c("bad1", "opt2"))) # Error: at least one bad match
#' try(g("opt1"))            # No error: not an element
#' try(g(c("opt1", "opt2"))) # No error, none are elements

#' @export
arg_element <- function(x, values,
                        .arg = rlang::caller_arg(x), .msg = NULL,
                        .call) {

  arg_supplied(values, .call = rlang::current_env()) |>
    internal_arg()

  if (is_null(x) || !all(is.element(x, values))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (is_null(values)) {
      err("{.arg {(.arg)}} cannot take on any value",
          .call = .call)
    }

    values <- unique(values)

    if (!any(nzchar(values))) {
      if (is_scalar(x)) {
        err('{.arg {(.arg)}} must be the empty string ({.val {""}})',
            .call = .call)
      }

      err('each element of {.arg {(.arg)}} must be the empty string ({.val {""}})',
          .call = .call)
    }

    one_of <- if (length(values) > 1L) "one of"

    if (length(x) <= 1L) {
      err("{.arg {(.arg)}} must be {one_of} {.or {.val {values}}}",
          .call = .call)
    }

    err("each element of {.arg {(.arg)}} must be {one_of} {.or {.val {values}}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_element
arg_not_element <- function(x, values,
                            .arg = rlang::caller_arg(x), .msg = NULL,
                            .call) {

  arg_supplied(values, .call = rlang::current_env()) |>
    internal_arg()

  if (is_not_null(x) && any(is.element(x, values))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    values <- unique(values)

    if (!any(nzchar(values))) {
      if (is_scalar(x)) {
        err('{.arg {(.arg)}} must not be the empty string ({.val {""}})',
            .call = .call)
      }

      err('no element of {.arg {(.arg)}} may be the empty string ({.val {""}})',
          .call = .call)
    }

    if (is_scalar(x)) {
      err("{.arg {(.arg)}} must not be {.or {.val {values}}}",
          .call = .call)
    }

    err("no element of {.arg {(.arg)}} may be {.or {.val {values}}}",
        .call = .call)
  }
}

#' Check Index Argument
#'
#' @description
#' Checks whether an argument is a valid column index (`arg_index()`) of a data set or a vector thereof (`arg_indices()`).
#'
#' @inheritParams arg_is
#' @param data a data set (i.e., a matrix or data frame)
#' @param .arg_data the name of the argument supplied to `data` to appear in error messages. The default is to extract the argument's name using [rlang::caller_arg()]. Ignored if `.msg` is supplied.
#'
#' @details
#' For `arg_indices()`, an error will be thrown unless one of the following are true:
#'
#' * `x` is a vector of counts (see [arg_counts()]) less than or equal to `ncol(data)`
#' * `x` is a character vector with values a subset of `colnames(data)`
#'
#' For `arg_index()`, `x` additionally must have length equal to 1. Passing `arg_index()` ensures that `data[, x]` (if `data` is a matrix) or `data[[x]]` (if `x` is a data frame) evaluate correctly.
#'
#' If `data` has no column names, an error will be thrown if `x` is a character vector.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_counts()], [arg_character()]
#'
#' @examples
#' dat <- data.frame(col1 = 1:5,
#'                   col2 = 6:10)
#'
#' f <- function(z) {
#'   arg_index(z, dat)
#' }
#'
#' try(f(1))         # No error
#' try(f(3))         # Error: not a valid index
#' try(f("col1"))    # No error
#' try(f("bad_col")) # Error: not a valid index
#' try(f(1:2))       # Error: arg_index() requires scalar
#'
#' mat <- matrix(1:9, ncol = 3)
#'
#' g <- function(z) {
#'   arg_indices(z, mat)
#' }
#'
#' try(g(1))     # No error
#' try(g(1:3))   # No error
#' try(g("col")) # Error: `mat` has no names

#' @export
arg_index <- function(x, data,
                      .arg = rlang::caller_arg(x),
                      .arg_data = rlang::caller_arg(data),
                      .msg = NULL, .call) {
  arg_supplied(data, .call = rlang::current_env()) |>
    internal_arg()
  arg_data(data, .call = rlang::current_env()) |>
    internal_arg()

  is_valid_num_index <- is_whole_numeric(x) && all(x >= 1L) && all(x <= ncol(data))

  is_valid_char_index <- !is_valid_num_index &&
    is_not_null(colnames(data)) &&
    is.character(x) &&
    all(x %in% colnames(data))

  if (!is_scalar(x) ||
      (!is_valid_num_index && !is_valid_char_index)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (is_null(colnames(data))) {
      err("{.arg {(.arg)}} must be the index of a column in {.arg {(.arg_data)}}",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be the name or index of a column in {.arg {(.arg_data)}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_index
arg_indices <- function(x, data,
                        .arg = rlang::caller_arg(x),
                        .arg_data = rlang::caller_arg(data),
                        .msg = NULL, .call) {
  arg_supplied(data, .call = rlang::current_env()) |>
    internal_arg()
  arg_data(data, .call = rlang::current_env()) |>
    internal_arg()

  is_valid_num_index <- is_whole_numeric(x) && all(x >= 1L) && all(x <= ncol(data))

  is_valid_char_index <- !is_valid_num_index &&
    is_not_null(colnames(data)) &&
    is.character(x) &&
    all(x %in% colnames(data))

  if (!is_valid_num_index && !is_valid_char_index) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (is_null(colnames(data))) {
      err("{.arg {(.arg)}} must be the {cli::qty(length(x))} ind{?ex/ices} of {?a/} column{?s} in {.arg {(.arg_data)}}",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be the {cli::qty(length(x))} name{?s} or ind{?ex/ices} of {?a/} column{?s} in {.arg {(.arg_data)}}",
        .call = .call)
  }
}

#' Check NA in Argument
#'
#' Checks whether an argument does not contain any `NA` values (`arg_no_NA()`), contains only `NA` values (`arg_all_NA()`), or is a scalar `NA` (`arg_is_NA()`).
#'
#' @inheritParams arg_is
#'
#' @details
#' `arg_no_NA()` throws an error when `anyNA(x)` is `0`. `arg_all_NA()` throws an error when `all(is.na(x))` is not `FALSE`. `arg_is_NA()` throws an error when `length(x)` is not 1 or `anyNA(x)` is `FALSE`.
#'
#' `arg_no_NA()` is useful for checking that a meaningful argument was supplied. `arg_all_NA()` and `arg_is_NA()` are primarily used for in [arg_or()] to denote that `NA` is an allowed argument.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_non_null()], [arg_supplied()], [anyNA()]
#'
#' @examples
#' f <- function(x) {
#'   arg_no_NA(x)  ## x must not be NA
#' }
#'
#' try(f(1))           ## No error
#' try(f(NA))          ## Error: x is NA
#' try(f(c(1, NA, 3))) ## Error: x contains NA
#'
#' f2 <- function(y) {
#'   arg_all_NA(y) ## y must be NA
#' }
#'
#' try(f2(NA))          ## No error
#' try(f2(c(NA, NA)))   ## No error
#' try(f2(1))           ## Error: y is not NA
#' try(f2(c(1, NA, 3))) ## Error: y is not all NA

#' @export
arg_no_NA <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                      .call) {
  if (anyNA(x)) {
    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (is_scalar(x)) {
      err("{.arg {(.arg)}} must not be {.val {NA}}",
          .call = .call)
    }

    err("{.arg {(.arg)}} must not contain {.val {NA}} values",
        .call = .call)
  }
}

#' @export
#' @rdname arg_no_NA
arg_is_NA <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                      .call) {
  if (!is_scalar(x) || !is.atomic(x) || !anyNA(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be {.val {NA}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_no_NA
arg_all_NA <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!all(is.na(x))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must only contain {.val {NA}} values",
        .call = .call)
  }
}

#' Check Common Argument Types
#' @name arg_data
#'
#' @description
#' Checks whether an argument is an atomic vector (`arg_atomic()`), a dimensionless atomic vector (`arg_vector()`), a [`list`] (`arg_list()`), a [data frame][data.frame] (`arg_data.frame()`), a [`matrix`] (`arg_matrix()`), an [`array`] (`arg_array()`), a rectangular data set (`arg_data()`), or an [`environment`] (`arg_env()`).
#'
#' @inheritParams arg_is
#' @param df_ok `logical`; whether to allow data frames (which are technically lists). Default is `FALSE` to throw an error on a data frame input.
#'
#' @details
#' Atomic vectors are checked using [is.atomic()]. Because matrices are considered atomic vectors, `arg_vector()` additionally checks that there is no `"dims"` attribute, i.e., that `length(dim(x)) == 0L`. `arg_data()` checks whether `x` is either a data frame or a matrix. `arg_list()` checks whether `x` is a list; when `df = FALSE`, it throws an error when `x` is a data frame, even though data frames are lists.
#'
#' @inherit arg_is return
#'
#' @seealso [is.atomic()], [is.list()], [is.data.frame()], [is.matrix()], [is.array()], [is.environment()], [arg_is()]
#'
#' @examples
#' vec <- 1:6
#' mat <- as.matrix(vec)
#' dat <- as.data.frame(mat)
#' lis <- as.list(vec)
#' nul <- NULL
#'
#' # arg_atomic
#' try(arg_atomic(vec))
#' try(arg_atomic(mat))
#' try(arg_atomic(dat))
#' try(arg_atomic(lis))
#' try(arg_atomic(nul))
#'
#' # arg_vector
#' try(arg_vector(vec))
#' try(arg_vector(mat))
#'
#' # arg_matrix
#' try(arg_matrix(vec))
#' try(arg_matrix(mat))
#' try(arg_matrix(dat))
#'
#' # arg_data.frame
#' try(arg_data.frame(vec))
#' try(arg_data.frame(mat))
#' try(arg_data.frame(dat))
#'
#' # arg_data
#' try(arg_data(vec))
#' try(arg_data(mat))
#' try(arg_data(dat))

#' @export
#' @rdname arg_data
arg_atomic <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is.atomic(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be an atomic vector",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_vector <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is.atomic(x) || !is_null(dim(x))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a vector",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_list <- function(x, df_ok = FALSE,
                     .arg = rlang::caller_arg(x), .msg = NULL,
                     .call) {
  arg_flag(df_ok, .call = rlang::current_env()) |>
    internal_arg()

  if (!is.list(x) || (is.data.frame(x) && !df_ok)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (is.data.frame(x)) {
      err("{.arg {(.arg)}} must be a list and not a data frame",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a list",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_data.frame <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                           .call) {
  if (!is.data.frame(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a data frame",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_matrix <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is.matrix(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a matrix",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_array <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                      .call) {
  if (!is.array(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be an array",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_data <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                     .call) {
  if (!is.data.frame(x) && !is.matrix(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a data frame or matrix",
        .call = .call)
  }
}

#' @export
#' @rdname arg_data
arg_env <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                    .call) {
  if (!is.environment(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be an enviornment",
        .call = .call)
  }
}

#' Argument Verification
#'
#' An alternative to [match.arg()] with improved
#' error messages via \pkg{cli}. Returns the first choice if `x` is `NULL`,
#' and supports partial matching.
#'
#' @inheritParams arg_is
#' @param x a string (or character vector if `several.ok = TRUE`) to match against `choices`. If `NULL`, the first element of `choices` is returned.
#' @param choices a character vector of valid values.
#' @param several.ok `logical`; if `TRUE`, `x` may contain more than one element. Defaults to `FALSE`.
#' @param ignore.case `logical`; if `FALSE`, the matching is case sensitive, and if `TRUE` (the default), case is ignored.
#' @param .context an optional character string providing context for error messages (e.g., a function or object name). Prepended to the error message when supplied before being evaluated by \pkg{cli}.
#'
#' @returns
#' A character string (or vector, if `several.ok = TRUE`) of the matched element(s) from `choices`. If there is no match, an error is thrown. When `several.ok = TRUE`, no error is thrown if there is at least one match.
#'
#' @details
#' Partial matching is supported via [pmatch()]. If no match is found, an error is thrown listing the valid `choices`.
#'
#' Checks are run on `x` prior to matching: first, [arg_supplied()] is used to check whether `x` was supplied; then [arg_string()] (if `several.ok = TRUE`) or [arg_character()] (if `several.ok = FALSE`) is used to check whether `x` is a valid string or character vector, respectively.
#'
#' When `ignore.case = TRUE` (the default), an initial case-sensitive match is run, and if any values in `x` are unmatched, a second, case-insensitive match is run. When `ignore.case = FALSE`, only the first match is run. This ensures that exact matches (on both content and case) are prioritized before case-insensitive matches.
#'
#' @seealso
#' * [match.arg()] for the base \R version
#' * [pmatch()] for the function implementing partial string matching
#' * [arg_element()] for a version without type checking and that doesn't support partial matching
#' * [rlang::arg_match()] for a similar \pkg{rlang} function that doesn't support partial matching.
#'
#' @examples
#' f <- function(z = NULL) {
#'   match_arg(z, c("None", "Exact", "Partial"),
#'             ignore.case = TRUE)
#' }
#'
#' try(f())            # "None" (first choice returned)
#' try(f("partial"))   # "Partial"
#' try(f("p"))         # "Partial" (partial match)
#' try(f(c("e", "p"))) # Error: several.ok = FALSE
#'
#' # several.ok = TRUE
#' g <- function(z = NULL) {
#'   match_arg(z, c("None", "Exact", "Partial"),
#'             several.ok = TRUE)
#' }
#'
#' try(g("exact"))               # Error: case not ignored
#' try(g("Exact"))               # "Exact"
#' try(g(c("Exact", "Partial"))) # "Exact", "Partial"
#' try(g(c("Exact", "Wrong")))   # "Exact"
#' try(g(c("Wrong1", "Wrong2"))) # Error: no match
#'
#' h <- function(z = NULL) {
#'   match_arg(z, c("None", "Exact", "Partial"),
#'             .context = "in {.fun h},")
#' }
#'
#' try(h("Wrong")) # Error with context

#' @export
match_arg <- function(x, choices, several.ok = FALSE, ignore.case = TRUE,
                      .context = NULL, .arg = rlang::caller_arg(x), .call) {
  arg_supplied(choices, .call = rlang::current_env()) |>
    internal_arg()
  arg_character(choices, .call = rlang::current_env()) |>
    internal_arg()

  arg_supplied(x, .arg = .arg, .call = .call)

  if (is_null(x)) {
    return(choices[1L])
  }

  arg_flag(several.ok, .call = rlang::current_env()) |>
    internal_arg()
  arg_flag(ignore.case, .call = rlang::current_env()) |>
    internal_arg()

  choices <- unique(choices)

  if (several.ok) {
    arg_character(x, .arg = .arg, .call = .call)
  }
  else {
    arg_string(x, .arg = .arg, .call = .call)
  }

  i <- pmatch(x, choices, nomatch = 0L,
              duplicates.ok = TRUE)

  if (ignore.case && any(i == 0)) {
    i[i == 0] <- pmatch(tolower(x[i == 0]), tolower(choices), nomatch = 0L,
                        duplicates.ok = TRUE)
  }

  if (all(i == 0L)) {
    one_of <- {
      if (length(choices) > 1L) {
        if (several.ok) "at least one of" else "one of"
      }
    }

    if (is_null(.context)) {
      err("{.arg {(.arg)}} should be {one_of} {.or {.val {choices}}}",
          .call = .call)
    }

    arg_string(.context, .call = rlang::current_env()) |>
      internal_arg()

    err(sprintf("%s {.arg {(.arg)}} should be {one_of} {.or {.val {choices}}}",
                .context),
        .call = .call)
  }

  i <- i[i > 0L]

  choices[i]
}

#' Check Argument Length
#'
#' @description
#' Checks whether an argument has a specified length.
#'
#' @inheritParams arg_is
#' @param len `integer`; the allowed length(s) of `x`. Default is 1.
#'
#' @details
#' `len` can contain multiple allowed counts; an error will be thrown only if `length(x)` is not equal to any value of `len`.
#'
#' @inherit arg_is return
#'
#' @seealso [length()], [arg_non_null()] to specifically test that an object's length is or is not 0.
#'
#' @examples
#' obj <- 1:4
#'
#' try(arg_length(obj, 1))
#' try(arg_length(obj, 4))
#' try(arg_length(obj, c(1, 4)))
#'
#' # These test the same thing:
#' try(arg_length(obj, c(0:3)))
#' try(when_not_null(obj,
#'                   arg_length(1:3)))

#' @export
arg_length <- function(x, len = 1L,
                       .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  arg_counts(len, .call = rlang::current_env()) |>
    internal_arg()

  if (!length(x) %in% len) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must have length {.or {len}}",
        .call = .call)
  }
}

#' Check Equal Arguments
#'
#' Checks whether two arguments are equal (`arg_equal()`) or not equal (`arg_not_equal()`).
#'
#' @inheritParams arg_is
#' @param x,x2 the objects to compare with each other.
#' @param .arg,.arg2 the names of the objects being compared; used in the error message if triggered. Ignored if `.msg` is supplied.
#' @param \dots other arguments passed to [all.equal()].
#'
#' @details
#' Tests for equality are performed by evaluating whether `isTRUE(all.equal(x, x2, ...))` is `TRUE` or `FALSE`.
#'
#' @inherit arg_is return
#'
#' @seealso [all.equal()]
#'
#' @examples
#' f <- function(x, y, ...) {
#'   arg_equal(x, y, ...)
#' }
#'
#' try(f(x = 1, y = 1))      ## No error
#' try(f(x = 1L, y = 1.0))   ## No error despite type difference
#' try(f(x = 1, y = 1.00001, ## No error, within tolerance
#'       tolerance = .001))
#'
#' try(f(x = 1, y = 2))       ## Error: different
#' try(f(x = 1, y = 1.00001)) ## Error
#'
#' g <- function(x, y, ...) {
#'   arg_not_equal(x, y, ...)
#' }
#'
#' try(g(x = 1, y = 1)) ## Error
#' try(g(x = 1, y = 2)) ## No error

#' @export
arg_equal <- function(x, x2, ...,
                      .arg = rlang::caller_arg(x),
                      .arg2 = rlang::caller_arg(x2),
                      .msg = NULL, .call) {
  arg_supplied(x2, .call = rlang::current_env()) |>
    internal_arg()

  if (!isTRUE(all.equal(x, x2, ...))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be equal to {.arg {(.arg2)}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_equal
arg_not_equal <- function(x, x2, ...,
                          .arg = rlang::caller_arg(x),
                          .arg2 = rlang::caller_arg(x2),
                          .msg = NULL, .call) {
  arg_supplied(x2, .call = rlang::current_env()) |>
    internal_arg()

  if (isTRUE(all.equal(x, x2, ...))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must not be equal to {.arg {(.arg2)}}",
        .call = .call)
  }
}

#' Check Supplied Dots Argument
#'
#' Checks whether an argument was supplied to `...`.
#'
#' @inheritParams arg_is
#' @param \dots the `...` argument passed from a calling function. The argument is not evaluated. See Examples.
#'
#' @details
#' `arg_dots_supplied()` checks whether arguments were passed to the `...` (i.e., "dots") argument of a function. These arguments are not evaluated. `arg_dots_supplied()` throws an error if [...length()] is 0. It can be useful for when a function must have additional arguments. For example, [arg_or()] and [when_supplied()] use [arg_dots_supplied()].
#'
#' `arg_dots_not_supplied()` checks whether no arguments were passed to `...`, again without evaluating the arguments and using `...length()`. It can be useful when a function appears to allow further arguments (e.g., because it is a method of a generic), but when the author does not want the user to supply them.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_supplied()]
#'
#' @examples
#' f <- function(...) {
#'   arg_dots_supplied(...)
#' }
#'
#' try(f(1)) # No error: argument supplied
#' try(f())  # Error!

#' @export
arg_dots_supplied <- function(..., .msg = NULL, .call) {
  if (...length() == 0L) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("an argument must be supplied to {.arg ...}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_dots_supplied
arg_dots_not_supplied <- function(..., .msg = NULL, .call) {
  if (...length() != 0L) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("no arguments may be supplied to {.arg ...}",
        .call = .call)
  }
}

#' Throw an Error, Warning, or Message
#'
#' These functions are similar to [stop()]/[cli::cli_abort()], [warning()]/[cli::cli_warn()], and [message()]/[cli::cli_inform()], throwing an error, warning, and message, respectively. Minor processing is done to capitalize the first letter of the message, add a period at the end (if it makes sense to), and add information about the calling function.
#'
#' @param m the message to be displayed, passed to the `message` argument of [rlang::abort()], [rlang::warn()], or [rlang::inform()].
#' @param .call the execution environment of a currently running function, e.g. `.call = rlang::current_env()`. The corresponding function call is retrieved and mentioned in error messages as the source of the error. See the `call` argument of [rlang::abort()] for details. Set to `NULL` to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.
#' @param .envir the environment to evaluate the glue expressions in. See [rlang::abort()] for details. Typically this does not need to be changed.
#' @param immediate whether to output the warning immediately (`TRUE`, the default) or save all warnings until the end of execution (`FALSE`). See [warning()] for details. Note that the default here differs from that of `warning()`.
#' @param \dots other arguments passed to [rlang::abort()], [rlang::warn()], or [rlang::inform()].
#'
#' @details
#' These functions are simple wrappers for the corresponding functions in \pkg{rlang}, namely [rlang::abort()] for `err()`, [rlang::warn()] for `wrn()`, and [rlang::inform()] for `msg()`, but which function almost identically to the \pkg{cli} versions. Their main differences are that they additionally process the input (capitalizing the first character of the message and adding a period to the end if needed, unless multiple strings are provided). `err()` is used inside all `arg_*()` functions in \pkg{arg}.
#'
#' @return
#' `err()` throws an error condition. `wrn()` throws a warning condition and invisibly returns the formatted warning message as a string. `msg()` signals a message and invisibly returns `NULL`.
#'
#' @seealso
#' * Base versions: [stop()], [warning()], [message()]
#' * \pkg{rlang} versions: [rlang::abort()], [rlang::warn()], [rlang::inform()]
#' * \pkg{cli} versions: [cli::cli_abort()], [cli::cli_warn()], [cli::cli_inform()]
#'
#' @examples
#' f <- function(x) {
#'   err("this is an error, and {.arg x} is {.type {x}}")
#' }
#'
#' try(f(1))
#'
#' g <- function(x) {
#'   wrn("this warning displayed last", immediate = FALSE)
#'   wrn("this warning displayed first")
#' }
#'
#' try(g(1))
#'
#' h <- function() {
#'   msg("is a period added at the end?")
#'   msg("not when the message ends in punctuation!")
#'   msg(c("or when multiple",
#'         "!" = "messages",
#'         "v" = "are",
#'         "*" = "displayed"))
#'   msg("otherwise yes")
#' }
#'
#' h()

#' @export
err <- function(m, .call, .envir = rlang::caller_env(), ...) {

  arg_env(.envir, .call = rlang::current_env()) |>
    internal_arg()

  if (rlang::is_missing(.call)) {
    .call <- .pkg_caller_call()
  }

  for (i in seq_along(m)) {
    m[i] <- cli::format_inline(m[i], .envir = .envir)
  }

  m[] <- .tidy_msg(m) |>
    cli::ansi_simplify()

  rlang::abort(m, call = .call, use_cli_format = TRUE,
               .frame = .envir, ...)
}

#' @export
#' @rdname err
wrn <- function(m, immediate = TRUE, .call, .envir = rlang::caller_env(), ...) {
  arg_flag(immediate, .call = rlang::current_env()) |>
    internal_arg()
  arg_env(.envir, .call = rlang::current_env()) |>
    internal_arg()

  if (rlang::is_missing(.call)) {
    .call <- .pkg_caller_call()
  }

  if (isTRUE(immediate) && isTRUE(all.equal(0, getOption("warn")))) {
    rlang::local_options(warn = 1)
  }

  for (i in seq_along(m)) {
    m[i] <- cli::format_inline(m[i], .envir = .envir)
  }

  m[] <- .tidy_msg(m) |>
    cli::ansi_simplify()

  cli::format_warning(m, .envir = .envir) |>
    rlang::warn(call = .call, ...)
}

#' @export
#' @rdname err
msg <- function(m, .call = NULL, .envir = rlang::caller_env(), ...) {
  arg_env(.envir, .call = rlang::current_env()) |>
    internal_arg()

  if (rlang::is_missing(.call)) {
    .call <- .pkg_caller_call()
  }

  for (i in seq_along(m)) {
    m[i] <- cli::format_inline(m[i], .envir = .envir)
  }

  m[] <- .tidy_msg(m) |>
    cli::ansi_simplify()

  cli::format_message(m, .envir = .envir) |>
    rlang::inform(call = .call, ...)
}

# Capitalize first character (if letter), add period at the end
# (if no other ending punctuation)
.tidy_msg <- function(m) {
  if (!is_scalar(m) || !nzchar(m)) {
    return(m)
  }

  # Capitalize first letter
  m[] <- ansi_upper_first(m)

  # Add period to end
  if (!cli::ansi_grepl("([.]|[?]|[!])$", m)) {
    m[] <- paste0(m, ".")
  }

  m
}

.pkg_caller_call <- function() {
  nf <- sys.nframe()

  arg <- utils::packageName()

  .pkg_names <- character(nf)
  pkg_name <- NULL

  # Get packahges names for all calls in the stack
  for (i in seq_len(nf)) {
    fn <- sys.function(i)

    if (!is.function(fn)) {
      next
    }

    env <- environment(fn)

    if (identical(env, globalenv())) {
      next
    }

    .pkg_names[i] <- environmentName(env)
  }

  .pkg_ignore <- c("", arg, "rlang", "base")

  if (all(.pkg_names %in% .pkg_ignore)) {
    return(NULL)
  }

  # Find block of arg, then block of other package; that
  # package's namespace is searched

  arg_passed <- FALSE
  fn_ind <- NULL
  pkg_name <- NULL

  for (i in seq(nf, 1L)) {
    if (!arg_passed) {
      if (.pkg_names[i] == arg) {
        arg_passed <- TRUE
      }

      next
    }

    if (is_null(fn_ind)) {
      if (!.pkg_names[i] %in% .pkg_ignore) {
        pkg_found <- TRUE
        fn_ind <- i
      }

      next
    }

    if (.pkg_names[i] != .pkg_names[fn_ind]) {
      break
    }

    fn_ind <- i
  }

  if (is_null(fn_ind)) {
    return(NULL)
  }

  pkg_name <- .pkg_names[fn_ind]

  pkg_funs <- c(getNamespaceExports(pkg_name),
                .getNamespaceInfo(asNamespace(pkg_name), "S3methods")[, 3L])

  if (is_null(pkg_funs)) {
    return(NULL)
  }

  e <- sys.call(fn_ind)

  n <- rlang::call_name(e)

  if (is_null(n) || !n %in% pkg_funs) {
    return(NULL)
  }

  e
}

is_null <- function(x) {identical(length(x), 0L)}
is_not_null <- function(x) {!is_null(x)}

is_scalar <- function(x) {identical(length(x), 1L)}

check_if_zero <- function(x, tolerance = sqrt(.Machine$double.eps)) {
  # this is the default tolerance used in all.equal
  abs(x) < tolerance
}

is_error <- function(x) {
  tryCatch(capture.output({out <- x}, type = "message"),
           error = function(e) {
             return(TRUE)
           }) |>
    suppressWarnings() |>
    suppressMessages() |>
    invisible()

  inherits(out, c("error", "try-error"))
}

is_whole_numeric <- function(x) {
  is_not_null(x) &&
    (is.integer(x) ||
       (is.numeric(x) && all(check_if_zero(x - trunc(x)))))
}

safe_any <- function(x) {
  x <- suppressWarnings(x)
  x[is.na(x)] <- FALSE
  any(x)
}
safe_all <- function(x) {
  x <- suppressWarnings(x)
  x[is.na(x)] <- FALSE
  all(x)
}

ansi_trim <- function(x, left = 0, right = 0) {
  n <- cli::ansi_nchar(x)

  cli::ansi_substr(x, left + 1L, n - right)
}
ansi_upper_first <- function(x) {
  starts_with_letters <- cli::ansi_grepl("^[[:alpha:]]", x)

  if (any(starts_with_letters)) {
    for (i in which(starts_with_letters)) {
      if (cli::ansi_nchar(x[i]) > 1L) {
        x[i] <- paste0(cli::ansi_toupper(cli::ansi_substr(x[i], 1L, 1L)),
                       cli::ansi_substr(x[i], 2L, cli::ansi_nchar(x[i])))

      }
      else {
        x[i] <- cli::ansi_toupper(x[i])
      }
    }
  }

  x
}
ansi_lower_first <- function(x) {
  starts_with_letters <- cli::ansi_grepl("^[[:alpha:]]", x)

  if (any(starts_with_letters)) {
    for (i in which(starts_with_letters)) {
      if (cli::ansi_nchar(x[i]) > 1L) {
        x[i] <- paste0(cli::ansi_tolower(cli::ansi_substr(x[i], 1L, 1L)),
                       cli::ansi_substr(x[i], 2L, cli::ansi_nchar(x[i])))

      }
      else {
        x[i] <- cli::ansi_tolower(x[i])
      }
    }
  }

  x
}

trim_final_punct <- function(x) {
  ends_with_punct <- cli::ansi_grepl("([.]|[?]|[!])$", x)

  if (any(ends_with_punct)) {
    x[ends_with_punct] <- ansi_trim(x[ends_with_punct], right = 1L)
  }

  x
}

any_apply <- function(X, FUN, ...) {
  FUN <- match.fun(FUN)
  if (!is.vector(X) || is.object(X)) {
    X <- as.list(X)
  }

  for (x in X) {
    if (isTRUE(FUN(x, ...))) {
      return(TRUE)
    }
  }

  FALSE
}
all_apply <- function(X, FUN, ...) {
  FUN <- match.fun(FUN)
  if (!is.vector(X) || is.object(X)) {
    X <- as.list(X)
  }

  for (x in X) {
    if (isFALSE(FUN(x, ...))) {
      return(FALSE)
    }
  }

  TRUE
}

# Makes an error an internal error, treated differently by
# and_or(), etc.
internal_arg <- function(expr) {
  rlang::try_fetch(expr,
                   error = function(e) {
                     class(e) <- c("internal_arg_error", class(e))
                     rlang::cnd_signal(e)
                   })
}

.msg_eval <- function(.msg, .call = rlang::caller_env()) {
  when_not_null(.msg,
                arg_character,
                .call = .call) |>
    internal_arg()

  .msg
}

#' Check Logical Argument
#'
#' @description
#' Checks whether an argument is a logical vector (`arg_logical()`) or a logical scalar (`arg_flag()`), i.e., a single logical value. Logical values include `TRUE` and `FALSE`.
#'
#' @inheritParams arg_is
#'
#' @details
#' `NA` values in `arg_flag()` will cause an error to be thrown.
#'
#' @inherit arg_is return
#'
#' @seealso [is.logical()], [rlang::is_bool()]
#'
#' @examples
#' obj <- TRUE
#'
#' try(arg_flag(obj))    # No error
#' try(arg_logical(obj)) # No error
#'
#' obj <- c(TRUE, FALSE)
#'
#' try(arg_flag(obj))    # Error: must be a scalar
#' try(arg_logical(obj)) # No error
#'
#' obj <- 1L
#'
#' try(arg_flag(obj))    # Error must be logical
#' try(arg_logical(obj)) # Error must be logical

#' @export
arg_logical <- function(x, .arg = rlang::caller_arg(x), .msg = NULL, .call) {
  if (!is.logical(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a logical vector",
        .call = .call)
  }
}

#' @export
#' @rdname arg_logical
arg_flag <- function(x, .arg = rlang::caller_arg(x), .msg = NULL, .call) {
  if (!is_scalar(x) || !is.logical(x) || anyNA(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_scalar(x)) {
      err("{.arg {(.arg)}} must be a single logical value ({.or {.val {c(TRUE, FALSE)}}})",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a logical value ({.or {.val {c(TRUE, FALSE)}}})",
        .call = .call)
  }
}

#' Check That Arguments Meet Some or All Criteria
#'
#' `arg_or()` checks whether an argument meets at least one given criterion. `arg_and()` checks whether an argument meets all given criteria.
#'
#' @inheritParams when_supplied
#' @inheritParams arg_is
#'
#' @details
#' For `arg_or()`, an error will be thrown only if `x` fails all of the supplied checks. This can be useful when an argument can take on one of several input types. For `arg_and()`, and error will be thrown if `x` fails any of the supplied checks. This is less useful on its own, as the checks can simply occur in sequence without `arg_and()`, but `arg_and()` can be supplied to the `...` argument of `arg_or()` to create more complicated criteria.
#'
#' The `...` arguments can be passed either as functions, e.g.,
#' ```
#' arg_or(x,
#'        arg_number,
#'        arg_string,
#'        arg_flag)
#' ```
#' or as unevaluated function calls with the `x` argument absent, e.g.,
#' ```
#' arg_or(x,
#'        arg_number(),
#'        arg_string(),
#'        arg_flag())
#' ```
#' or as a mixture of both.
#'
#' These functions do their best to provide a clean error message composed of all the error messages for the failed checks. With many options, this can yield a complicated error message, so use caution. `arg_and()` marks with a check (```r cli::symbol$tick```) any passed checks and with a cross (```r cli::symbol$cross```) any failed checks.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_supplied()], [when_not_null()]
#'
#' @examples
#' # `arg_or()`
#' f <- function(z) {
#'   arg_or(z,
#'          arg_number,
#'          arg_string,
#'          arg_flag)
#' }
#'
#' try(f(1))      # No error
#' try(f("test")) # No error
#' try(f(TRUE))   # No error
#' try(f(1:4))    # Error: neither a number, string,
#' #              #        or flag, but a vector
#'
#' # `arg_and()`
#' g <- function(z) {
#'   arg_and(z,
#'           arg_counts,
#'           arg_length(len = 2),
#'           arg_lt(bound = 5))
#' }
#'
#' try(g(c(1, 2)))     # No error
#' try(g(c(1, 7)))     # Error: not < 5
#' try(g(c(1.1, 2.1))) # Error: not counts
#' try(g(4))           # Error: not length 2
#' try(g("bad"))       # Error: no criteria satisfied
#'
#' # Chaining together `arg_and()` and `arg_or()`
#' h <- function(z) {
#'   arg_or(z,
#'          arg_all_NA,
#'          arg_and(arg_count,
#'                  arg_lt(5)),
#'          arg_and(arg_string,
#'                  arg_element(c("a", "b", "c"))))
#' }
#'
#' try(h(NA))  # No error
#' try(h(1))   # No error
#' try(h("a")) # No error
#' try(h(7))   # Error: not < 5
#' try(h("d")) # Error: not in "a", "b", or "c"

#' @export
arg_or <- function(x, ..., .arg = rlang::caller_arg(x), .msg = NULL, .call) {
  force(.arg)

  if (...length() == 0L) {
    return(invisible())
  }

  dots <- rlang::call_match(dots_expand = FALSE)[["..."]]

  failures <- character(...length())

  x_name <- rlang::caller_arg(x)

  for (i in seq_along(dots)) {
    cnd <- .to_arg_fun_call(dots[[i]], x_name, .arg) |>
      eval.parent() |>
      rlang::catch_cnd()

    if (!inherits(cnd, "error")) {
      return(invisible())
    }

    if (rlang::cnd_inherits(cnd, "internal_arg_error")) {
      err("", .call = rlang::current_env(), parent = cnd)
    }

    failures[i] <- conditionMessage(cnd)
  }

  if (is_not_null(.msg)) {
    err(.msg_eval(.msg), .call = .call)
  }

  grouped_msgs <- group_messages(failures, "or")
  grouped <- attr(grouped_msgs, "grouped")

  for (i in which(!grouped)) {
    msg_i <- failures[i]

    if (cli::ansi_grepl("^at least one of the following conditions must be met:", msg_i,
                        ignore.case = TRUE)) {

      msg_i <- Reduce(function(m, sy) unlist(strsplit(m, sy, TRUE)),
                      as.list(unlist(cli::symbol)),
                      init = msg_i) |>
        unlist() |>
        cli::ansi_trimws()

      grouped_msgs_i <- group_messages(msg_i[-1L], "or")
      grouped_i <- attr(grouped_msgs_i, "grouped")

      grouped_msgs <- c(grouped_msgs,
                        grouped_msgs_i,
                        msg_i[-1L][!grouped_i] |>
                          cli::ansi_trimws() |>
                          trim_final_punct() |>
                          ansi_lower_first() |>
                          cli::ansi_collapse())
    }
    else if (cli::ansi_grepl("^all of the following conditions must be met:", msg_i,
                             ignore.case = TRUE)) {

      msg_i <- Reduce(function(m, sy) unlist(strsplit(m, sy, TRUE)),
                      as.list(unlist(cli::symbol)),
                      init = msg_i) |>
        unlist() |>
        cli::ansi_trimws()

      grouped_msgs_i <- group_messages(msg_i[-1L], "and")
      grouped_i <- attr(grouped_msgs_i, "grouped")

      grouped_msgs <- c(grouped_msgs,
                        grouped_msgs_i,
                        msg_i[-1L][!grouped_i] |>
                          cli::ansi_trimws() |>
                          trim_final_punct() |>
                          ansi_lower_first() |>
                          cli::ansi_collapse(", and "))
    }
    else {
      grouped_msgs <- c(grouped_msgs,
                        msg_i |>
                          cli::ansi_trimws() |>
                          trim_final_punct() |>
                          ansi_lower_first())
    }
  }

  n_err <- sum(nzchar(grouped_msgs))

  if (n_err == 1L) {
    err(grouped_msgs[nzchar(grouped_msgs)],
        .call = .call)
  }

  err(c("At least one of the following conditions must be met:",
        setNames(grouped_msgs[nzchar(grouped_msgs)],
                 rep.int("*", n_err))),
      .call = .call)
}

#' @export
#' @rdname arg_or
arg_and <- function(x, ..., .arg = rlang::caller_arg(x), .msg = NULL, .call) {
  force(.arg)

  dots <- rlang::call_match(dots_expand = FALSE)[["..."]]

  failures <- character(...length())
  failed <- rep.int(FALSE, ...length())

  x_name <- rlang::caller_arg(x)

  for (i in seq_along(dots)) {
    cnd <- .to_arg_fun_call(dots[[i]], x_name, .arg) |>
      eval.parent() |>
      rlang::catch_cnd()

    if (rlang::cnd_inherits(cnd, "internal_arg_error")) {
      err("", .call = rlang::current_env(), parent = cnd)
    }

    if (inherits(cnd, "error")) {
      failures[i] <- conditionMessage(cnd)
      failed[i] <- TRUE
    }
  }

  if (!any(failed)) {
    return(invisible())
  }

  if (is_not_null(.msg)) {
    err(.msg_eval(.msg), .call = .call)
  }

  ## Get error messages for non-failures
  for (i in which(!failed)) {
    cnd <- .to_arg_fun_call(dots[[i]], x_name, .arg) |>
      make_fail(x) |>
      eval.parent() |>
      rlang::catch_cnd()

    if (inherits(cnd, "error")) {
      failures[i] <- conditionMessage(cnd)
    }
  }

  dup <- duplicated(failures) | !nzchar(failures)
  failures <- failures[!dup]
  failed <- failed[!dup]

  for (i in seq_along(failures)) {
    msg_i <- failures[i]

    if (cli::ansi_grepl("^at least one of the following conditions must be met:", msg_i,
                        ignore.case = TRUE)) {

      msg_i <- Reduce(function(m, sy) unlist(strsplit(m, sy, TRUE)),
                      as.list(unlist(cli::symbol)),
                      init = msg_i) |>
        unlist() |>
        cli::ansi_trimws()

      failures[i] <- msg_i[-1L] |>
        cli::ansi_trimws() |>
        trim_final_punct() |>
        ansi_lower_first() |>
        cli::ansi_collapse(last = ", or ")
    }
    else if (cli::ansi_grepl("^all of the following conditions must be met:", msg_i,
                             ignore.case = TRUE)) {

      msg_i <- Reduce(function(m, sy) unlist(strsplit(m, sy, TRUE)),
                      as.list(unlist(cli::symbol)),
                      init = msg_i) |>
        unlist() |>
        cli::ansi_trimws()

      failures[i] <- msg_i[-1L] |>
        cli::ansi_trimws() |>
        trim_final_punct() |>
        ansi_lower_first() |>
        cli::ansi_collapse(", and ")
    }
    else {
      failures[i] <- msg_i |>
        trim_final_punct() |>
        cli::ansi_trimws()
    }
  }

  if (is_scalar(failures)) {
    err(failures, .call = .call)
  }

  err(c("All of the following conditions must be met:",
        setNames(failures, ifelse(failed, "x", "v"))),
      .call = .call)
}

group_messages <- function(messages, and_or = "and", .envir = parent.frame()) {
  prefixes <- .prefixes()
  grouped_msgs <- character(length(prefixes))
  grouped <- rep(FALSE, length(messages))

  for (i in seq_along(prefixes)) {
    prefix <- cli::format_inline(prefixes[i], .envir = .envir)
    starts_with_prefix <- cli::ansi_grepl(paste0("^", prefix), messages[!grouped])

    if (sum(starts_with_prefix) < 2L) {
      next
    }

    if (endsWith(prefix, " a")) {
      offset <- rep(0:1, c(1L, sum(starts_with_prefix) - 1L))
    }
    else {
      offset <- 0
    }

    grouped_msgs[i] <- paste(prefix,
                             messages[!grouped][starts_with_prefix] |>
                               ansi_trim(cli::ansi_nchar(prefix) - offset) |>
                               trim_final_punct() |>
                               cli::ansi_trimws() |>
                               cli::ansi_collapse(last = sprintf(", %s ", and_or)))

    grouped[!grouped][starts_with_prefix] <- TRUE

    if (all(grouped)) {
      break
    }
  }

  attr(grouped_msgs, "grouped") <- grouped

  grouped_msgs
}

.prefixes <- function() {
  p <- c(
    #"{.arg {(.arg)}} must be a",
    "{.arg {(.arg)}} must inherit from",
    "{.arg {(.arg)}} must be",
    "{.arg {(.arg)}} must have",
    # "{.arg {(.arg)}} must",
    "{.arg {(.arg)}}"
    )

  c(p, paste("each element of", p))
}

#' Check Numeric Argument
#'
#' @description
#' Checks whether an argument is numeric, with some additional constraints if desired. `arg_numeric()` simply checks whether the argument is numeric. `arg_number()` checks whether the argument is a numeric scalar (i.e., a single number). `arg_whole_numeric()` and `arg_whole_number()` check whether the argument is a whole numeric vector or scalar, respectively. `arg_counts()` and `arg_count()` check whether the argument is a non-negative whole numeric vector or scalar, respectively.
#'
#' @inheritParams arg_is
#'
#' @details
#' A whole number is decided by testing whether the value is an integer (i.e., using [is.integer()]) or if `abs(x - trunc(x)) < sqrt(.Machine$double.eps)`. This is the same tolerance used by [all.equal()] to compare values.
#'
#' @inherit arg_is return
#'
#' @seealso [is.integer()], [rlang::is_integerish()]
#'
#' @examples
#' count <- 3
#' whole <- -4
#' num <- .943
#'
#' try(arg_number(count))       # No error
#' try(arg_whole_number(count)) # No error
#' try(arg_count(count))        # No error
#'
#' try(arg_number(whole))       # No error
#' try(arg_whole_number(whole)) # No error
#' try(arg_count(whole))        # Error: negatives not allowed
#'
#' try(arg_number(num))       # No error
#' try(arg_whole_number(num)) # Error: not a whole number
#' try(arg_count(num))        # Error: not a count
#'
#' nums <- c(0, .5, 1)
#'
#' try(arg_number(nums))  # Error: not a single number
#' try(arg_numeric(nums)) # No error
#' try(arg_counts(nums))  # Error: not counts

#' @export
arg_numeric <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                        .call) {
  if (!is.numeric(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a numeric vector",
        .call = .call)
  }
}

#' @export
#' @rdname arg_numeric
arg_number <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is_scalar(x) || !is.numeric(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_scalar(x)) {
      err("{.arg {(.arg)}} must be a single number",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a number",
        .call = .call)
  }
}

#' @export
#' @rdname arg_numeric
arg_whole_numeric <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                              .call) {
  if (!is_whole_numeric(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a whole numeric vector",
        .call = .call)
  }
}

#' @export
#' @rdname arg_numeric
arg_whole_number <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                             .call) {
  if (!is_scalar(x) || !is_whole_numeric(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_scalar(x)) {
      err("{.arg {(.arg)}} must be a single whole number",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a whole number",
        .call = .call)
  }
}

#' @export
#' @rdname arg_numeric
arg_counts <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (!is_whole_numeric(x) || any(x < 0)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a vector of counts (non-negative whole numeric values)",
        .call = .call)
  }
}

#' @export
#' @rdname arg_numeric
arg_count <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                      .call) {
  if (!is_scalar(x) || !is_whole_numeric(x) ||
      any(x < 0)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (!is_scalar(x)) {
      err("{.arg {(.arg)}} must be a single count (a non-negative whole number)",
          .call = .call)
    }

    err("{.arg {(.arg)}} must be a count (a non-negative whole number)",
        .call = .call)
  }
}

#' Check Argument Class
#'
#' @description
#' Checks whether an argument is a of a specified class (`arg_is()`) or is not of a specified class (`arg_is_not()`).
#'
#' @param x the argument to be checked
#' @param class a character vector of one or more classes to check `x` against.
#' @param .arg the name of the argument supplied to `x` to appear in error messages. The default is to extract the argument's name using [rlang::caller_arg()]. Ignored if `.msg` is supplied.
#' @param .msg an optional alternative message to display if an error is thrown instead of the default message.
#' @param .call the execution environment of a currently running function, e.g. `.call = rlang::current_env()`. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to [err()]. Set to `NULL` to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.
#'
#' @details
#' `arg_is()` and `arg_is_not()` use [inherits()] to test for class membership. `arg_is()` throws an error only if no elements of `class(x)` are in `class`. `arg_is_not()` throws an error if any elements of `class(x)` are in `class`.
#'
#' Sometimes this can be too permissive; combining `arg_is()` with [arg_and()] can be useful for requiring that an object is of multiple classes. Alternatively, combining `arg_is_not()` with [arg_or()] can be useful for requiring that an object is not a specific combination of classes. See Examples.
#'
#' @return
#' Returns `NULL` invisibly if an error is not thrown.
#'
#' @seealso [inherits()], [arg_or()]; [arg_data()] for testing for specific kinds of objects (vectors, matrices, data frames, and lists)
#'
#' @examples
#' obj <- structure(list(1),
#'                  class = "test")
#'
#' try(arg_is(obj, "test"))            # No error
#' try(arg_is(obj, c("test", "quiz"))) # No error
#' try(arg_is(obj, "quiz"))            # Error
#'
#' try(arg_is_not(obj, "test"))            # Error
#' try(arg_is_not(obj, c("test", "quiz"))) # Error
#' try(arg_is_not(obj, "quiz"))            # No error
#'
#' # Multiple classes
#' obj2 <- structure(list(1),
#'                   class = c("test", "essay"))
#'
#' try(arg_is(obj2, c("test", "quiz")))     # No error
#' try(arg_is_not(obj2, c("test", "quiz"))) # Error
#'
#' ## Require argument to be of multiple classes
#' try(arg_and(obj2,
#'             arg_is("test"),
#'             arg_is("quiz")))
#'
#' ## Require argument to not be a specific combination of
#' ## multiple classes
#' try(arg_or(obj2,
#'            arg_is_not("test"),
#'            arg_is_not("essay")))

#' @export
arg_is <- function(x, class,
                   .arg = rlang::caller_arg(x), .msg = NULL,
                   .call) {
  arg_supplied(class, .call = rlang::current_env()) |>
    internal_arg()
  arg_character(class, .call = rlang::current_env()) |>
    internal_arg()

  if (!inherits(x, class)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must inherit from class {.or {.cls {class}}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_is
arg_is_not <- function(x, class,
                       .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  arg_supplied(class, .call = rlang::current_env()) |>
    internal_arg()
  arg_character(class, .call = rlang::current_env()) |>
    internal_arg()

  if (inherits(x, class)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must not inherit from class {.or {.cls {class}}}",
        .call = .call)
  }
}

#' Check Named Argument
#'
#' @description
#' Checks whether an argument has valid (non-`NULL`, non-empty, and non-`NA`) names.
#'
#' @inheritParams arg_is
#'
#' @details
#' `arg_named()` works for vectors, lists, and data frames. To check whether a matrix has column names, use `arg_colnamed()` (which also works for data frames, but not vectors or lists).
#'
#' @inherit arg_is return
#'
#' @seealso [rlang::is_named()], [names()], [colnames()], [arg_data()]
#'
#' @examples
#' obj <- c(1,
#'          B = 2,
#'          C = 3)
#'
#' try(arg_named(obj)) # Error: one name is blank
#'
#' names(obj)[1L] <- "A"
#' try(arg_named(obj)) # No error
#'
#' obj2 <- unname(obj)
#' try(arg_named(obj2)) # Error: no names
#'
#' # Matrix and data frame
#' mat <- matrix(1:6, ncol = 2L)
#' colnames(mat) <- c("A", "B")
#'
#' try(arg_named(mat))    # Error: matrices are not named
#' try(arg_colnamed(mat)) # No error
#'
#' dat <- as.data.frame(mat)
#' try(arg_named(dat))    # No error: data frames are named
#' try(arg_colnamed(dat)) # No error
#'
#' colnames(mat) <- NULL
#' try(arg_colnamed(mat)) # Error: no colnames

#' @export
arg_named <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                      .call) {
  if (is_null(names(x)) || anyNA(names(x)) ||
      !all(nzchar(names(x)))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must have non-empty and non-{.val {NA}} names",
        .call = .call)
  }
}

#' @export
#' @rdname arg_named
arg_colnamed <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                         .call) {
  if (is_null(colnames(x)) || anyNA(colnames(x)) ||
      !all(nzchar(colnames(x)))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must have non-empty and non-{.val {NA}} names",
        .call = .call)
  }
}

#' Check Formula Argument
#'
#' @description
#' Checks whether an argument is a [`formula`].
#'
#' @inheritParams arg_is
#' @param one_sided `NULL` or `logical`; if `TRUE`, checks that `x` is a `formula` with only one side (the right side); if `FALSE`, checks that `x` is a `formula` with both sides; if `NULL` (the default), checks only that `x` is a `formula.`
#'
#' @inherit arg_is return
#'
#' @seealso [rlang::is_formula()], [arg_is()]
#'
#' @examples
#' form1 <- ~a + b
#' form2 <- y ~ a + b
#' not_form <- 1:3
#'
#' try(arg_formula(form1))    # No error
#' try(arg_formula(form2))    # No error
#' try(arg_formula(not_form)) # Error: not a formula
#'
#' try(arg_formula(form1,
#'                 one_sided = TRUE)) # No error
#' try(arg_formula(form2,
#'                 one_sided = TRUE)) # Error, not one-sided
#'
#' try(arg_formula(form1,
#'                 one_sided = FALSE)) # Error, only one-sided
#' try(arg_formula(form2,
#'                 one_sided = FALSE)) # No error

#' @export
arg_formula <- function(x, one_sided = NULL,
                        .arg = rlang::caller_arg(x),
                        .msg = NULL, .call) {

  when_not_null(one_sided, arg_flag,
                .call = rlang::current_env()) |>
    internal_arg()

  if ((is_null(one_sided) && !rlang::is_formula(x)) ||
      (is_not_null(one_sided) && !rlang::is_formula(x, lhs = !one_sided))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    if (is_null(one_sided)) {
      err("{.arg {(.arg)}} must be a formula",
          .call = .call)
    }

    sides <- if (one_sided) "one" else "two"
    err("{.arg {(.arg)}} must be a {sides}-sided formula",
        .call = .call)
  }
}

#' Check Arguments When Supplied
#'
#' These functions check arguments only when they are supplied (`when_supplied()`) or when not `NULL` (`when_not_null()`). Multiple checks can be applied in sequence. This allows arguments not to have to be supplied, but checks them only if they are.
#'
#' @inheritParams arg_is
#' @param \dots `arg_*()` functions or unevaluated function calls to be applied to `x`. See Examples.
#'
#' @details
#' An error will be thrown only if `x` is supplied and fails one of the supplied checks (`when_supplied()`) or is not `NULL` and fails one of the supplied checks (`when_not_null()`).
#'
#' The `...` arguments can be passed either as functions, e.g.,
#' ```
#' when_supplied(x,
#'               arg_number,
#'               arg_gt)
#' ```
#' or as unevaluated function calls with the `x` argument absent, e.g.,
#' ```
#' when_supplied(x,
#'               arg_number(),
#'               arg_gt(bound = 0))
#' ```
#' or as a mixture of both.
#'
#' `when_supplied()` only makes sense to use for an argument that has no default value but which can be omitted. `when_not_null()` makes sense to use for an argument with a default value of `NULL`.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_or()], [arg_supplied()]
#'
#' @examples
#' f <- function(z) {
#'   when_supplied(z,
#'                 arg_number,
#'                 arg_between(c(0, 1)))
#' }
#'
#' try(f())    # No error: not supplied
#' try(f("a")) # Error: not a number
#' try(f(2))   # Error: not within 0-1 range
#' try(f(.7))  # No error: number within range
#'
#' g <- function(z = NULL) {
#'   when_not_null(z,
#'                 arg_number,
#'                 arg_between(c(0, 1)))
#' }
#'
#' try(g())     # No error: NULL okay
#' try(g(NULL)) # No error: NULL okay
#' try(g("a"))  # Error: not a number
#' try(g(2))    # Error: not within 0-1 range
#' try(g(.7))   # No error: number within range

#' @export
when_supplied <- function(x, ..., .arg = rlang::caller_arg(x), .call) {
  if (!rlang::is_missing(x)) {
    force(.arg)
    arg_dots_supplied(..., .call = rlang::current_env()) |>
      internal_arg()

    dots <- rlang::call_match(dots_expand = FALSE)[["..."]]
    x_name <- rlang::caller_arg(x)

    for (i in seq_along(dots)) {
      cnd <- .to_arg_fun_call(dots[[i]], x_name, .arg) |>
        eval.parent() |>
        rlang::catch_cnd()

      if (rlang::cnd_inherits(cnd, "internal_arg_error")) {
        err("", .call = rlang::current_env(), parent = cnd)
      }

      if (inherits(cnd, "error")) {
        .msg <- conditionMessage(cnd) |>
          cli::ansi_strsplit("\n") |>
          unlist(recursive = FALSE)

        .msg[1L] <- paste("When {.arg {(.arg)}} is supplied,", ansi_lower_first(.msg[1L]))

        err(.msg, .call = .call)
      }
    }
  }
}

#' @export
#' @rdname when_supplied
when_not_null <- function(x, ..., .arg = rlang::caller_arg(x), .call) {
  if (is_not_null(x)) {
    force(.arg)
    arg_dots_supplied(..., .call = rlang::current_env()) |>
      internal_arg()

    dots <- rlang::call_match(dots_expand = FALSE)[["..."]]
    x_name <- rlang::caller_arg(x)

    for (i in seq_along(dots)) {
      cnd <- .to_arg_fun_call(dots[[i]], x_name, .arg) |>
        eval.parent() |>
        rlang::catch_cnd()

      if (rlang::cnd_inherits(cnd, "internal_arg_error")) {
        err("", .call = rlang::current_env(), parent = cnd)
      }

      if (inherits(cnd, "error")) {
        .msg <- conditionMessage(cnd) |>
          cli::ansi_strsplit("\n") |>
          unlist(recursive = FALSE)

        .msg[1L] <- paste(cli::format_inline("When {.arg {(.arg)}} is not {.val {list(NULL)}},"),
                          ansi_lower_first(.msg[1L]))

        err(.msg, .call = .call)
      }
    }
  }
}

.to_arg_fun_call <- function(arg_call, x_name, .arg) {
  if (!is.call(arg_call) ||
      any_apply(as.list(arg_call), identical, as.name("::")) ||
      any_apply(as.list(arg_call), identical, as.name(":::"))) {
    arg_call <- rlang::call2(arg_call)
  }

  call_fun <- eval.parent(arg_call[[1L]])

  fmls <- rlang::fn_fmls_names(call_fun)

  if ("x" %in% fmls) {
    arg_call <- rlang::call_modify(arg_call, !!!list(x = str2lang(x_name)))
  }

  if (".arg" %in% fmls) {
    arg_call <- rlang::call_modify(arg_call, !!!list(.arg = .arg))
  }

  if (".call" %in% fmls) {
    arg_call <- rlang::call_modify(arg_call, !!!list(.call = NULL))
  }

  rlang::call_match(call = arg_call, fn = call_fun)
}

#' Check Unique Argument
#'
#' @description
#' Checks whether an argument contains only unique values.
#'
#' @inheritParams arg_is
#' @param \dots further argument passed to [anyDuplicated()], which performs the checking.
#'
#' @inherit arg_is return
#'
#' @seealso [anyDuplicated()]
#'
#' @examples
#' f <- function(z) {
#'   arg_unique(z)
#' }
#'
#' try(f(1:3))     # No error
#' try(f(NULL))    # No error for NULL
#' try(f(c(1, 1))) # Error: repeated values

#' @export
arg_unique <- function(x, ..., .arg = rlang::caller_arg(x), .msg = NULL,
                       .call) {
  if (is_not_null(x) && anyDuplicated(x, ...) != 0) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must only contain unique values",
        .call = .call)
  }
}

#' Check NULL Argument
#'
#' Checks whether an argument is non-`NULL` (`arg_non_null()`) or is `NULL` (`arg_null()`). `arg_non_null()` throws an error when `length(x)` is `0`. `arg_null()` throws an error when `length(x)` is not `0`.
#'
#' @inheritParams arg_is
#'
#' @details
#' Here, `NULL` refers to any length-0 object, including `NULL`, `logical(0L)`, `list()`, `3[FALSE]`, etc. `arg_non_null()` is useful for checking that a meaningful argument was supplied. `arg_null()` is primarily used for in [arg_or()] to denote that `NULL` is an allowed argument.
#'
#' @inherit arg_is return
#'
#' @seealso [arg_length()], [arg_no_NA()], [arg_supplied()]
#'
#' @examples
#' f <- function(x = NULL, y = NULL) {
#'   arg_non_null(x) ## x must not be NULL
#'   arg_null(y)     ## y must be NULL
#' }
#'
#' try(f(x = 1,    y = NULL)) ## No error
#' try(f(x = NULL, y = NULL)) ## Error: x is NULL
#' try(f(x = 1,    y = 1))    ## Error: y is non-NULL
#'
#' # Any object of length 0 is considered NULL
#' try(f(x = numeric())) ## Error: x is NULL
#' try(f(x = list()))    ## Error: x is NULL
#'
#' test <- c(1, 2)[c(FALSE, FALSE)]
#' try(f(x = test))      ## Error: x is NULL
#'
#' # arg_null() is best used in and_or():
#' f2 <- function(z) {
#'   arg_or(z,
#'          arg_null(),
#'          arg_number())
#' }
#'
#' try(f2(NULL)) ## No error; z can be NULL
#' try(f2(1))    ## No error; z can be a number
#' try(f2(TRUE)) ## Error: z must be NULL or a number

#' @export
arg_non_null <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                         .call) {
  if (is_null(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be non-{.val {list(NULL)}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_non_null
arg_null <- function(x, .arg = rlang::caller_arg(x), .msg = NULL,
                     .call) {
  if (is_not_null(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be {.val {list(NULL)}}",
        .call = .call)
  }
}

#' Check Values in Range
#'
#' Checks whether values are within a range (`arg_between()`), greater than some value (`arg_gt()`), greater than or equal to some value (`arg_gte()`), less than some value (`arg_lt()`), or less than or equal to some value (`arg_lte()`).
#'
#' @inheritParams arg_is
#' @param range a vector of two values identifying the required range.
#' @param inclusive a logical vector identifying whether the range boundaries are inclusive or not. If only one value is supplied, it is applied to both range boundaries. Default is `TRUE`, which will not throw an error if `x` is equal to the boundaries.
#' @param bound the bound to check against. Default is 0.
#'
#' @details
#' `x` is not checked for type, as it is possible for values other than numeric values to be passed and compared; however, an error will be thrown if `typeof(x)` is not equal to `typeof(range)` or `typeof(bound)`. `NA` values of `x` will be removed. The arguments to `range`, `inclusive`, and `bound` are checked for appropriateness.
#'
#' @inherit arg_is return
#'
#' @examples
#' z <- 2
#'
#' try(arg_between(z, c(1, 3))) # No error
#' try(arg_between(z, c(1, 2))) # No error
#' try(arg_between(z, c(1, 2),
#'                 inclusive = FALSE)) # Error
#' try(arg_between(z, c(1, 2),
#'                 inclusive = c(TRUE, FALSE))) # Error
#'
#' try(arg_gt(z, 0))  # No error
#' try(arg_gt(z, 2))  # Error
#' try(arg_gte(z, 2)) # No error
#'
#' try(arg_lt(z, 0))  # Error
#' try(arg_lt(z, 2))  # Error
#' try(arg_lte(z, 2)) # No error
#'
#' try(arg_lte(z, "3")) # Error: wrong type

#' @export
arg_between <- function(x, range = c(0, 1), inclusive = TRUE,
                        .arg = rlang::caller_arg(x), .msg = NULL,
                        .call) {

  arg_length(range, 2L, .call = rlang::current_env()) |>
    internal_arg()
  arg_logical(inclusive, .call = rlang::current_env()) |>
    internal_arg()
  arg_length(inclusive, 1:2, .call = rlang::current_env()) |>
    internal_arg()

  if (is_scalar(inclusive)) {
    inclusive <- inclusive[c(1L, 1L)]
  }

  if (is.unsorted(range)) {
    o_range <- order(range)
    range <- range[o_range]
    inclusive <- inclusive[o_range]
  }

  .gt_comp <- if (inclusive[1L]) function(a, b) {a >= b} else function(a, b) {a > b}
  .lt_comp <- if (inclusive[2L]) function(a, b) {a <= b} else function(a, b) {a < b}

  if (!are_comparable(x, range[1L], .gt_comp) ||
      !are_comparable(x, range[2L], .lt_comp) ||
      !safe_all(.gt_comp(x, range[1L])) ||
      !safe_all(.lt_comp(x, range[2L]))) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    each_element_of <- if (length(x) > 1L) "each element of"

    if (all(inclusive)) {
      err("{each_element_of} {.arg {(.arg)}} must be between {.val {range}} (inclusive)",
          .call = .call)
    }

    if (!any(inclusive)) {
      err("{each_element_of} {.arg {(.arg)}} must be between {.val {range}} (exclusive)",
          .call = .call)
    }

    .gt_str <- {
      if (range[1L] == 0 && !inclusive[1L]) "positive"
      else if (inclusive[1L]) "greater than or equal to {.val {range[1L]}}"
      else "greater than {.val {range[1L]}}"
    }

    .lt_str <- {
      if (range[2L] == 0 && !inclusive[2L]) "negative"
      else if (inclusive[2L]) "less than or equal to {.val {range[2L]}}"
      else "less than {.val {range[2L]}}"
    }

    err(sprintf("{each_element_of} {.arg {(.arg)}} must be %s and %s",
                .gt_str, .lt_str),
        .call = .call)
  }
}

#' @export
#' @rdname arg_between
arg_gt <- function(x, bound = 0,
                   .arg = rlang::caller_arg(x), .msg = NULL,
                   .call) {
  arg_length(bound, 1L, .call = rlang::current_env()) |>
    internal_arg()

  if (!are_comparable(x, bound, `>`) ||
      !safe_all(x > bound)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    each_element_of <- if (length(x) > 1L) "each element of"

    if (bound == 0) {
      err("{each_element_of} {.arg {(.arg)}} must be positive",
          .call = .call)
    }

    err("{each_element_of} {.arg {(.arg)}} must be greater than {.val {bound}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_between
arg_gte <- function(x, bound = 0,
                    .arg = rlang::caller_arg(x), .msg = NULL,
                    .call) {
  arg_length(bound, 1L, .call = rlang::current_env()) |>
    internal_arg()

  if (!are_comparable(x, bound, `>=`) ||
      !safe_all(x >= bound)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    each_element_of <- if (length(x) > 1L) "each element of"

    err("{each_element_of} {.arg {(.arg)}} must be greater than or equal to {.val {bound}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_between
arg_lt <- function(x, bound = 0,
                   .arg = rlang::caller_arg(x), .msg = NULL,
                   .call) {
  arg_length(bound, 1L, .call = rlang::current_env()) |>
    internal_arg()

  if (!are_comparable(x, bound, `<`) ||
      !safe_all(x < bound)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    each_element_of <- if (length(x) > 1L) "each element of"

    if (bound == 0) {
      err("{each_element_of} {.arg {(.arg)}} must be negative",
          .call = .call)
    }

    err("{each_element_of} {.arg {(.arg)}} must be less than {.val {bound}}",
        .call = .call)
  }
}

#' @export
#' @rdname arg_between
arg_lte <- function(x, bound = 0,
                    .arg = rlang::caller_arg(x), .msg = NULL,
                    .call) {
  arg_length(bound, 1L, .call = rlang::current_env()) |>
    internal_arg()

  if (!are_comparable(x, bound, `<=`) ||
      !safe_all(x <= bound)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    each_element_of <- if (length(x) > 1L) "each element of"

    # if (bound == 0) {
    #   err("{each_element_of} {.arg {(.arg)}} must be non-positive",
    #       .call = .call)
    # }

    err("{each_element_of} {.arg {(.arg)}} must be less than or equal to {.val {bound}}",
        .call = .call)
  }
}

are_comparable <- function(x, y, comparison) {
  if (is.numeric(x)) {
    return(is.numeric(y))
  }

  if (is.character(x)) {
    return(is.character(y))
  }

  if (is.numeric(y) || is.character(y)) {
    return(FALSE)
  }

  tryCatch({
    comparison(x, y)
    TRUE
  },
  error = function(e) FALSE,
  warning = function(w) FALSE)
}

make_fail <- function(a0, x) {
  fn <- rlang::call_name(a0)

  a <- match.call(match.fun(fn), a0, expand.dots = FALSE)

  args <- rlang::call_args(a)
  for (i in which(names(args) != "x")) {
    args[[i]] <- eval.parent(args[[i]])
  }

  if (fn %in% c("arg_and", "arg_or")) {
    new_args <- lapply(args[["..."]], function(j) {
      j <- .to_arg_fun_call(j)

      if ("x" %in% names(j)) {
        j <- rlang::call_modify(j, !!!list(x = args[["x"]]))
      }
      if (".arg" %in% names(j)) {
        j <- rlang::call_modify(j, !!!list(.arg = args[[".arg"]]))
      }
      if (".call" %in% names(j)) {
        j <- rlang::call_modify(j, !!!list(.call = NULL))
      }

      make_fail(j, x)
    })

    a[names(a) == "..."] <- NULL

    a <- rlang::call_modify(a, !!!new_args)

    return(a)
  }

  new_args <-  switch(
    fn,
    arg_between = {
      list(x = rep(NaN, 2L))
    },
    arg_dots_not_supplied = {
      list("..." = list(1L))
    },
    arg_dots_supplied = {
      list("..." = rlang::zap())
    },
    arg_element = {
      if (anyNA(args[["values"]])) {
        b <- 1
        while (is.element(b, args[["values"]])) b <- b + 1
        list(x = rep.int(b, 2L))
      }
      else {
        list(x = rep.int(NA, 2L))
      }
    },
    arg_equal = {
      if (isTRUE(all.equal(x, NaN))) list(x = 1L)
      else list(x = NaN)
    },
    arg_gt = {
      list(x = rep(NaN, 2L))
    },
    arg_gte = {
      list(x = rep(NaN, 2L))
    },
    arg_index = {
      list(x = rep.int(NaN, 2L))
    },
    arg_indices = {
      list(x = rep.int(NaN, 2L))
    },
    arg_is = {
      list(x = structure(list(), class = paste(args[["class"]], collapse = "")))
    },
    arg_is_not = {
      list(x = structure(list(), class = args[["class"]]))
    },
    arg_length = {
      list(x = rep.int(x, max(args[["len"]] + 1L)))
    },
    arg_list = {
      list(x = 1L)
    },
    arg_lt = {
      list(x = rep(NaN, 2L))
    },
    arg_lte = {
      list(x = rep(NaN, 2L))
    },
    arg_no_NA = {
      list(x = rep.int(NA, 2L))
    },
    arg_non_null = {
      list(x = NULL)
    },
    arg_not_element = {
      list(x = rep.int(args[["values"]][1L], 2L))
    },
    arg_not_equal = {
      list(x = args[["x2"]])
    },
    # arg_or,
    arg_supplied = {
      list(x = rlang::missing_arg())
    },
    {
      list(x = .evil_object)
    })

  rlang::call_modify(a, !!!new_args)
}

.evil_object <- list("5d1561a3444c04149260df7165afc077db2ab5a7",
                     844337346787.926008639,
                     NA, 1, 1) |>
  setNames(c("e7bf8d80193c2fd16935c965c174db395b9824e4", NA, "hehe", "a", "b")) |>
  structure(class = "arg_evil_object")

#' Check Function Argument
#'
#' @description
#' Checks whether an argument is a function.
#'
#' @inheritParams arg_is
#'
#' @inherit arg_is return
#'
#' @seealso [rlang::is_function()], [arg_is()]
#'
#' @examples
#' f <- function(z) {
#'   arg_function(z)
#' }
#'
#' try(f(print))   # No error
#' try(f("print")) # Error: must be a function

#' @export
arg_function <- function(x, .arg = rlang::caller_arg(x), .msg = NULL, .call) {
  if (!rlang::is_function(x)) {

    if (is_not_null(.msg)) {
      err(.msg_eval(.msg), .call = .call)
    }

    err("{.arg {(.arg)}} must be a function",
        .call = .call)
  }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_function.R
\name{arg_function}
\alias{arg_function}
\title{Check Function Argument}
\usage{
arg_function(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is a function.
}
\examples{
f <- function(z) {
  arg_function(z)
}

try(f(print))   # No error
try(f("print")) # Error: must be a function
}
\seealso{
\code{\link[rlang:is_function]{rlang::is_function()}}, \code{\link[=arg_is]{arg_is()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_between.R
\name{arg_between}
\alias{arg_between}
\alias{arg_gt}
\alias{arg_gte}
\alias{arg_lt}
\alias{arg_lte}
\title{Check Values in Range}
\usage{
arg_between(
  x,
  range = c(0, 1),
  inclusive = TRUE,
  .arg = rlang::caller_arg(x),
  .msg = NULL,
  .call
)

arg_gt(x, bound = 0, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_gte(x, bound = 0, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_lt(x, bound = 0, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_lte(x, bound = 0, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{range}{a vector of two values identifying the required range.}

\item{inclusive}{a logical vector identifying whether the range boundaries are inclusive or not. If only one value is supplied, it is applied to both range boundaries. Default is \code{TRUE}, which will not throw an error if \code{x} is equal to the boundaries.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}

\item{bound}{the bound to check against. Default is 0.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether values are within a range (\code{arg_between()}), greater than some value (\code{arg_gt()}), greater than or equal to some value (\code{arg_gte()}), less than some value (\code{arg_lt()}), or less than or equal to some value (\code{arg_lte()}).
}
\details{
\code{x} is not checked for type, as it is possible for values other than numeric values to be passed and compared; however, an error will be thrown if \code{typeof(x)} is not equal to \code{typeof(range)} or \code{typeof(bound)}. \code{NA} values of \code{x} will be removed. The arguments to \code{range}, \code{inclusive}, and \code{bound} are checked for appropriateness.
}
\examples{
z <- 2

try(arg_between(z, c(1, 3))) # No error
try(arg_between(z, c(1, 2))) # No error
try(arg_between(z, c(1, 2),
                inclusive = FALSE)) # Error
try(arg_between(z, c(1, 2),
                inclusive = c(TRUE, FALSE))) # Error

try(arg_gt(z, 0))  # No error
try(arg_gt(z, 2))  # Error
try(arg_gte(z, 2)) # No error

try(arg_lt(z, 0))  # Error
try(arg_lt(z, 2))  # Error
try(arg_lte(z, 2)) # No error

try(arg_lte(z, "3")) # Error: wrong type
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_symmetric.R
\name{arg_symmetric}
\alias{arg_symmetric}
\alias{arg_cov}
\alias{arg_cor}
\alias{arg_distance}
\title{Check Symmetric Matrix Argument}
\usage{
arg_symmetric(
  x,
  tol = 100 * .Machine$double.eps,
  ...,
  .arg = rlang::caller_arg(x),
  .msg = NULL,
  .call
)

arg_cov(
  x,
  tol = 100 * .Machine$double.eps,
  ...,
  .arg = rlang::caller_arg(x),
  .msg = NULL,
  .call
)

arg_cor(
  x,
  tol = 100 * .Machine$double.eps,
  ...,
  .arg = rlang::caller_arg(x),
  .msg = NULL,
  .call
)

arg_distance(
  x,
  tol = 100 * .Machine$double.eps,
  ...,
  .arg = rlang::caller_arg(x),
  .msg = NULL,
  .call
)
}
\arguments{
\item{x}{the argument to be checked}

\item{tol}{\code{numeric}; the tolerance value used to assess symmetry and any numerical bounds. Passed to \code{\link[=isSymmetric]{isSymmetric()}}.}

\item{\dots}{other arguments passed to \code{\link[=isSymmetric]{isSymmetric()}} (and eventually to \code{\link[=all.equal]{all.equal()}}).}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
\code{arg_symmetric()} checks whether an argument is a square, symmetric, numeric matrix. \code{arg_cov()} checks whether an argument is like a covariance matrix (i.e., square and symmetric with non-negative diagonal entries). \code{arg_cor()} checks whether an argument is like a correlation matrix (i.e., square and symmetric with all values between -1 and 1 and ones on the diagonal). \code{arg_distance()} checks whether an argument is like a distance matrix (i.e., square and symmetric with non-negative entries and zeros on the diagonal).
}
\details{
These functions check that an argument is a square, symmetric, numeric matrix. This can be useful when a function can accept a covariance matrix, correlation matrix, or distance matrix. \code{arg_cov()} will throw an error if any values on its diagonal are less than \code{-tol}. \code{arg_cor()} will throw an error if any of its values are greater than \code{1 + tol} in absolute value or if the diagonal entries are not between \code{-1 - tol} and \code{1 + tol}. \code{arg_distance()} will thrown an error if any of its values are less than \code{-tol} or if the diagonal entries are not between \code{-tol} and \code{tol}. The tolerance is just slightly greater than 0 to allow for numeric imprecision.

No checks on semi-definiteness or rank are performed.
}
\examples{
set.seed(1234)
mat <- matrix(rnorm(12), ncol = 3L) # Non square
sym_mat <- -crossprod(mat)          # Square, symmetric
cov_mat <- cov(mat)                 # Covariance
cor_mat <- cor(mat)                 # Correlation
dist_mat <- as.matrix(dist(mat))    # Distance

try(arg_symmetric(mat))     # Error: not square
try(arg_symmetric(sym_mat)) # No error

try(arg_cov(sym_mat)) # Error: diagonal must be non-negative
try(arg_cov(cov_mat)) # No error

try(arg_cor(cov_mat)) # Error: values must be in [-1, 1]
try(arg_cor(cor_mat)) # No error

try(arg_distance(cor_mat))  # Error: diagonal must be 0
try(arg_distance(dist_mat)) # No error
}
\seealso{
\code{\link[=arg_numeric]{arg_numeric()}}, \code{\link[=arg_matrix]{arg_matrix()}}, \code{\link[=arg_between]{arg_between()}}, \code{\link[=isSymmetric]{isSymmetric()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_character.R
\name{arg_character}
\alias{arg_character}
\alias{arg_string}
\alias{arg_factor}
\title{Check Character Argument}
\usage{
arg_character(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_string(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_factor(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is a character vector (\code{arg_character()}), a character scalar (\code{arg_string()}), or a factor (\code{arg_factor()}).
}
\details{
\code{NA} values in \code{arg_string()} will cause an error to be thrown.
}
\examples{
f <- function(z) {
  arg_string(z)
}

try(f("a")) # No error
try(f(c("a", "b"))) # Error: arg_string() requires scalar
try(f(NA)) # NAs not allowed for arg_string()
}
\seealso{
\code{\link[=is.character]{is.character()}}, \code{\link[=is.factor]{is.factor()}}, \code{\link[rlang:is_string]{rlang::is_string()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_is.R
\name{arg_is}
\alias{arg_is}
\alias{arg_is_not}
\title{Check Argument Class}
\usage{
arg_is(x, class, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_is_not(x, class, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{class}{a character vector of one or more classes to check \code{x} against.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is a of a specified class (\code{arg_is()}) or is not of a specified class (\code{arg_is_not()}).
}
\details{
\code{arg_is()} and \code{arg_is_not()} use \code{\link[=inherits]{inherits()}} to test for class membership. \code{arg_is()} throws an error only if no elements of \code{class(x)} are in \code{class}. \code{arg_is_not()} throws an error if any elements of \code{class(x)} are in \code{class}.

Sometimes this can be too permissive; combining \code{arg_is()} with \code{\link[=arg_and]{arg_and()}} can be useful for requiring that an object is of multiple classes. Alternatively, combining \code{arg_is_not()} with \code{\link[=arg_or]{arg_or()}} can be useful for requiring that an object is not a specific combination of classes. See Examples.
}
\examples{
obj <- structure(list(1),
                 class = "test")

try(arg_is(obj, "test"))            # No error
try(arg_is(obj, c("test", "quiz"))) # No error
try(arg_is(obj, "quiz"))            # Error

try(arg_is_not(obj, "test"))            # Error
try(arg_is_not(obj, c("test", "quiz"))) # Error
try(arg_is_not(obj, "quiz"))            # No error

# Multiple classes
obj2 <- structure(list(1),
                  class = c("test", "essay"))

try(arg_is(obj2, c("test", "quiz")))     # No error
try(arg_is_not(obj2, c("test", "quiz"))) # Error

## Require argument to be of multiple classes
try(arg_and(obj2,
            arg_is("test"),
            arg_is("quiz")))

## Require argument to not be a specific combination of
## multiple classes
try(arg_or(obj2,
           arg_is_not("test"),
           arg_is_not("essay")))
}
\seealso{
\code{\link[=inherits]{inherits()}}, \code{\link[=arg_or]{arg_or()}}; \code{\link[=arg_data]{arg_data()}} for testing for specific kinds of objects (vectors, matrices, data frames, and lists)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_formula.R
\name{arg_formula}
\alias{arg_formula}
\title{Check Formula Argument}
\usage{
arg_formula(
  x,
  one_sided = NULL,
  .arg = rlang::caller_arg(x),
  .msg = NULL,
  .call
)
}
\arguments{
\item{x}{the argument to be checked}

\item{one_sided}{\code{NULL} or \code{logical}; if \code{TRUE}, checks that \code{x} is a \code{formula} with only one side (the right side); if \code{FALSE}, checks that \code{x} is a \code{formula} with both sides; if \code{NULL} (the default), checks only that \code{x} is a \code{formula.}}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is a \code{\link{formula}}.
}
\examples{
form1 <- ~a + b
form2 <- y ~ a + b
not_form <- 1:3

try(arg_formula(form1))    # No error
try(arg_formula(form2))    # No error
try(arg_formula(not_form)) # Error: not a formula

try(arg_formula(form1,
                one_sided = TRUE)) # No error
try(arg_formula(form2,
                one_sided = TRUE)) # Error, not one-sided

try(arg_formula(form1,
                one_sided = FALSE)) # Error, only one-sided
try(arg_formula(form2,
                one_sided = FALSE)) # No error
}
\seealso{
\code{\link[rlang:is_formula]{rlang::is_formula()}}, \code{\link[=arg_is]{arg_is()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_numeric.R
\name{arg_numeric}
\alias{arg_numeric}
\alias{arg_number}
\alias{arg_whole_numeric}
\alias{arg_whole_number}
\alias{arg_counts}
\alias{arg_count}
\title{Check Numeric Argument}
\usage{
arg_numeric(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_number(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_whole_numeric(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_whole_number(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_counts(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_count(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is numeric, with some additional constraints if desired. \code{arg_numeric()} simply checks whether the argument is numeric. \code{arg_number()} checks whether the argument is a numeric scalar (i.e., a single number). \code{arg_whole_numeric()} and \code{arg_whole_number()} check whether the argument is a whole numeric vector or scalar, respectively. \code{arg_counts()} and \code{arg_count()} check whether the argument is a non-negative whole numeric vector or scalar, respectively.
}
\details{
A whole number is decided by testing whether the value is an integer (i.e., using \code{\link[=is.integer]{is.integer()}}) or if \code{abs(x - trunc(x)) < sqrt(.Machine$double.eps)}. This is the same tolerance used by \code{\link[=all.equal]{all.equal()}} to compare values.
}
\examples{
count <- 3
whole <- -4
num <- .943

try(arg_number(count))       # No error
try(arg_whole_number(count)) # No error
try(arg_count(count))        # No error

try(arg_number(whole))       # No error
try(arg_whole_number(whole)) # No error
try(arg_count(whole))        # Error: negatives not allowed

try(arg_number(num))       # No error
try(arg_whole_number(num)) # Error: not a whole number
try(arg_count(num))        # Error: not a count

nums <- c(0, .5, 1)

try(arg_number(nums))  # Error: not a single number
try(arg_numeric(nums)) # No error
try(arg_counts(nums))  # Error: not counts
}
\seealso{
\code{\link[=is.integer]{is.integer()}}, \code{\link[rlang:is_integerish]{rlang::is_integerish()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match_arg.R
\name{match_arg}
\alias{match_arg}
\title{Argument Verification}
\usage{
match_arg(
  x,
  choices,
  several.ok = FALSE,
  ignore.case = TRUE,
  .context = NULL,
  .arg = rlang::caller_arg(x),
  .call
)
}
\arguments{
\item{x}{a string (or character vector if \code{several.ok = TRUE}) to match against \code{choices}. If \code{NULL}, the first element of \code{choices} is returned.}

\item{choices}{a character vector of valid values.}

\item{several.ok}{\code{logical}; if \code{TRUE}, \code{x} may contain more than one element. Defaults to \code{FALSE}.}

\item{ignore.case}{\code{logical}; if \code{FALSE}, the matching is case sensitive, and if \code{TRUE} (the default), case is ignored.}

\item{.context}{an optional character string providing context for error messages (e.g., a function or object name). Prepended to the error message when supplied before being evaluated by \pkg{cli}.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
A character string (or vector, if \code{several.ok = TRUE}) of the matched element(s) from \code{choices}. If there is no match, an error is thrown. When \code{several.ok = TRUE}, no error is thrown if there is at least one match.
}
\description{
An alternative to \code{\link[=match.arg]{match.arg()}} with improved
error messages via \pkg{cli}. Returns the first choice if \code{x} is \code{NULL},
and supports partial matching.
}
\details{
Partial matching is supported via \code{\link[=pmatch]{pmatch()}}. If no match is found, an error is thrown listing the valid \code{choices}.

Checks are run on \code{x} prior to matching: first, \code{\link[=arg_supplied]{arg_supplied()}} is used to check whether \code{x} was supplied; then \code{\link[=arg_string]{arg_string()}} (if \code{several.ok = TRUE}) or \code{\link[=arg_character]{arg_character()}} (if \code{several.ok = FALSE}) is used to check whether \code{x} is a valid string or character vector, respectively.

When \code{ignore.case = TRUE} (the default), an initial case-sensitive match is run, and if any values in \code{x} are unmatched, a second, case-insensitive match is run. When \code{ignore.case = FALSE}, only the first match is run. This ensures that exact matches (on both content and case) are prioritized before case-insensitive matches.
}
\examples{
f <- function(z = NULL) {
  match_arg(z, c("None", "Exact", "Partial"),
            ignore.case = TRUE)
}

try(f())            # "None" (first choice returned)
try(f("partial"))   # "Partial"
try(f("p"))         # "Partial" (partial match)
try(f(c("e", "p"))) # Error: several.ok = FALSE

# several.ok = TRUE
g <- function(z = NULL) {
  match_arg(z, c("None", "Exact", "Partial"),
            several.ok = TRUE)
}

try(g("exact"))               # Error: case not ignored
try(g("Exact"))               # "Exact"
try(g(c("Exact", "Partial"))) # "Exact", "Partial"
try(g(c("Exact", "Wrong")))   # "Exact"
try(g(c("Wrong1", "Wrong2"))) # Error: no match

h <- function(z = NULL) {
  match_arg(z, c("None", "Exact", "Partial"),
            .context = "in {.fun h},")
}

try(h("Wrong")) # Error with context
}
\seealso{
\itemize{
\item \code{\link[=match.arg]{match.arg()}} for the base \R version
\item \code{\link[=pmatch]{pmatch()}} for the function implementing partial string matching
\item \code{\link[=arg_element]{arg_element()}} for a version without type checking and that doesn't support partial matching
\item \code{\link[rlang:arg_match]{rlang::arg_match()}} for a similar \pkg{rlang} function that doesn't support partial matching.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/err.R
\name{err}
\alias{err}
\alias{wrn}
\alias{msg}
\title{Throw an Error, Warning, or Message}
\usage{
err(m, .call, .envir = rlang::caller_env(), ...)

wrn(m, immediate = TRUE, .call, .envir = rlang::caller_env(), ...)

msg(m, .call = NULL, .envir = rlang::caller_env(), ...)
}
\arguments{
\item{m}{the message to be displayed, passed to the \code{message} argument of \code{\link[rlang:abort]{rlang::abort()}}, \code{\link[rlang:warn]{rlang::warn()}}, or \code{\link[rlang:inform]{rlang::inform()}}.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. See the \code{call} argument of \code{\link[rlang:abort]{rlang::abort()}} for details. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}

\item{.envir}{the environment to evaluate the glue expressions in. See \code{\link[rlang:abort]{rlang::abort()}} for details. Typically this does not need to be changed.}

\item{\dots}{other arguments passed to \code{\link[rlang:abort]{rlang::abort()}}, \code{\link[rlang:warn]{rlang::warn()}}, or \code{\link[rlang:inform]{rlang::inform()}}.}

\item{immediate}{whether to output the warning immediately (\code{TRUE}, the default) or save all warnings until the end of execution (\code{FALSE}). See \code{\link[=warning]{warning()}} for details. Note that the default here differs from that of \code{warning()}.}
}
\value{
\code{err()} throws an error condition. \code{wrn()} throws a warning condition and invisibly returns the formatted warning message as a string. \code{msg()} signals a message and invisibly returns \code{NULL}.
}
\description{
These functions are similar to \code{\link[=stop]{stop()}}/\code{\link[cli:cli_abort]{cli::cli_abort()}}, \code{\link[=warning]{warning()}}/\code{\link[cli:cli_warn]{cli::cli_warn()}}, and \code{\link[=message]{message()}}/\code{\link[cli:cli_inform]{cli::cli_inform()}}, throwing an error, warning, and message, respectively. Minor processing is done to capitalize the first letter of the message, add a period at the end (if it makes sense to), and add information about the calling function.
}
\details{
These functions are simple wrappers for the corresponding functions in \pkg{rlang}, namely \code{\link[rlang:abort]{rlang::abort()}} for \code{err()}, \code{\link[rlang:warn]{rlang::warn()}} for \code{wrn()}, and \code{\link[rlang:inform]{rlang::inform()}} for \code{msg()}, but which function almost identically to the \pkg{cli} versions. Their main differences are that they additionally process the input (capitalizing the first character of the message and adding a period to the end if needed, unless multiple strings are provided). \code{err()} is used inside all \verb{arg_*()} functions in \pkg{arg}.
}
\examples{
f <- function(x) {
  err("this is an error, and {.arg x} is {.type {x}}")
}

try(f(1))

g <- function(x) {
  wrn("this warning displayed last", immediate = FALSE)
  wrn("this warning displayed first")
}

try(g(1))

h <- function() {
  msg("is a period added at the end?")
  msg("not when the message ends in punctuation!")
  msg(c("or when multiple",
        "!" = "messages",
        "v" = "are",
        "*" = "displayed"))
  msg("otherwise yes")
}

h()
}
\seealso{
\itemize{
\item Base versions: \code{\link[=stop]{stop()}}, \code{\link[=warning]{warning()}}, \code{\link[=message]{message()}}
\item \pkg{rlang} versions: \code{\link[rlang:abort]{rlang::abort()}}, \code{\link[rlang:warn]{rlang::warn()}}, \code{\link[rlang:inform]{rlang::inform()}}
\item \pkg{cli} versions: \code{\link[cli:cli_abort]{cli::cli_abort()}}, \code{\link[cli:cli_warn]{cli::cli_warn()}}, \code{\link[cli:cli_inform]{cli::cli_inform()}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_data.R
\name{arg_data}
\alias{arg_data}
\alias{arg_atomic}
\alias{arg_vector}
\alias{arg_list}
\alias{arg_data.frame}
\alias{arg_matrix}
\alias{arg_array}
\alias{arg_env}
\title{Check Common Argument Types}
\usage{
arg_atomic(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_vector(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_list(x, df_ok = FALSE, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_data.frame(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_matrix(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_array(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_data(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_env(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}

\item{df_ok}{\code{logical}; whether to allow data frames (which are technically lists). Default is \code{FALSE} to throw an error on a data frame input.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is an atomic vector (\code{arg_atomic()}), a dimensionless atomic vector (\code{arg_vector()}), a \code{\link{list}} (\code{arg_list()}), a \link[=data.frame]{data frame} (\code{arg_data.frame()}), a \code{\link{matrix}} (\code{arg_matrix()}), an \code{\link{array}} (\code{arg_array()}), a rectangular data set (\code{arg_data()}), or an \code{\link{environment}} (\code{arg_env()}).
}
\details{
Atomic vectors are checked using \code{\link[=is.atomic]{is.atomic()}}. Because matrices are considered atomic vectors, \code{arg_vector()} additionally checks that there is no \code{"dims"} attribute, i.e., that \code{length(dim(x)) == 0L}. \code{arg_data()} checks whether \code{x} is either a data frame or a matrix. \code{arg_list()} checks whether \code{x} is a list; when \code{df = FALSE}, it throws an error when \code{x} is a data frame, even though data frames are lists.
}
\examples{
vec <- 1:6
mat <- as.matrix(vec)
dat <- as.data.frame(mat)
lis <- as.list(vec)
nul <- NULL

# arg_atomic
try(arg_atomic(vec))
try(arg_atomic(mat))
try(arg_atomic(dat))
try(arg_atomic(lis))
try(arg_atomic(nul))

# arg_vector
try(arg_vector(vec))
try(arg_vector(mat))

# arg_matrix
try(arg_matrix(vec))
try(arg_matrix(mat))
try(arg_matrix(dat))

# arg_data.frame
try(arg_data.frame(vec))
try(arg_data.frame(mat))
try(arg_data.frame(dat))

# arg_data
try(arg_data(vec))
try(arg_data(mat))
try(arg_data(dat))
}
\seealso{
\code{\link[=is.atomic]{is.atomic()}}, \code{\link[=is.list]{is.list()}}, \code{\link[=is.data.frame]{is.data.frame()}}, \code{\link[=is.matrix]{is.matrix()}}, \code{\link[=is.array]{is.array()}}, \code{\link[=is.environment]{is.environment()}}, \code{\link[=arg_is]{arg_is()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_logical.R
\name{arg_logical}
\alias{arg_logical}
\alias{arg_flag}
\title{Check Logical Argument}
\usage{
arg_logical(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_flag(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is a logical vector (\code{arg_logical()}) or a logical scalar (\code{arg_flag()}), i.e., a single logical value. Logical values include \code{TRUE} and \code{FALSE}.
}
\details{
\code{NA} values in \code{arg_flag()} will cause an error to be thrown.
}
\examples{
obj <- TRUE

try(arg_flag(obj))    # No error
try(arg_logical(obj)) # No error

obj <- c(TRUE, FALSE)

try(arg_flag(obj))    # Error: must be a scalar
try(arg_logical(obj)) # No error

obj <- 1L

try(arg_flag(obj))    # Error must be logical
try(arg_logical(obj)) # Error must be logical
}
\seealso{
\code{\link[=is.logical]{is.logical()}}, \code{\link[rlang:is_bool]{rlang::is_bool()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_index.R
\name{arg_index}
\alias{arg_index}
\alias{arg_indices}
\title{Check Index Argument}
\usage{
arg_index(
  x,
  data,
  .arg = rlang::caller_arg(x),
  .arg_data = rlang::caller_arg(data),
  .msg = NULL,
  .call
)

arg_indices(
  x,
  data,
  .arg = rlang::caller_arg(x),
  .arg_data = rlang::caller_arg(data),
  .msg = NULL,
  .call
)
}
\arguments{
\item{x}{the argument to be checked}

\item{data}{a data set (i.e., a matrix or data frame)}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.arg_data}{the name of the argument supplied to \code{data} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is a valid column index (\code{arg_index()}) of a data set or a vector thereof (\code{arg_indices()}).
}
\details{
For \code{arg_indices()}, an error will be thrown unless one of the following are true:
\itemize{
\item \code{x} is a vector of counts (see \code{\link[=arg_counts]{arg_counts()}}) less than or equal to \code{ncol(data)}
\item \code{x} is a character vector with values a subset of \code{colnames(data)}
}

For \code{arg_index()}, \code{x} additionally must have length equal to 1. Passing \code{arg_index()} ensures that \code{data[, x]} (if \code{data} is a matrix) or \code{data[[x]]} (if \code{x} is a data frame) evaluate correctly.

If \code{data} has no column names, an error will be thrown if \code{x} is a character vector.
}
\examples{
dat <- data.frame(col1 = 1:5,
                  col2 = 6:10)

f <- function(z) {
  arg_index(z, dat)
}

try(f(1))         # No error
try(f(3))         # Error: not a valid index
try(f("col1"))    # No error
try(f("bad_col")) # Error: not a valid index
try(f(1:2))       # Error: arg_index() requires scalar

mat <- matrix(1:9, ncol = 3)

g <- function(z) {
  arg_indices(z, mat)
}

try(g(1))     # No error
try(g(1:3))   # No error
try(g("col")) # Error: `mat` has no names
}
\seealso{
\code{\link[=arg_counts]{arg_counts()}}, \code{\link[=arg_character]{arg_character()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_dots_supplied.R
\name{arg_dots_supplied}
\alias{arg_dots_supplied}
\alias{arg_dots_not_supplied}
\title{Check Supplied Dots Argument}
\usage{
arg_dots_supplied(..., .msg = NULL, .call)

arg_dots_not_supplied(..., .msg = NULL, .call)
}
\arguments{
\item{\dots}{the \code{...} argument passed from a calling function. The argument is not evaluated. See Examples.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument was supplied to \code{...}.
}
\details{
\code{arg_dots_supplied()} checks whether arguments were passed to the \code{...} (i.e., "dots") argument of a function. These arguments are not evaluated. \code{arg_dots_supplied()} throws an error if \code{\link[=...length]{...length()}} is 0. It can be useful for when a function must have additional arguments. For example, \code{\link[=arg_or]{arg_or()}} and \code{\link[=when_supplied]{when_supplied()}} use \code{\link[=arg_dots_supplied]{arg_dots_supplied()}}.

\code{arg_dots_not_supplied()} checks whether no arguments were passed to \code{...}, again without evaluating the arguments and using \code{...length()}. It can be useful when a function appears to allow further arguments (e.g., because it is a method of a generic), but when the author does not want the user to supply them.
}
\examples{
f <- function(...) {
  arg_dots_supplied(...)
}

try(f(1)) # No error: argument supplied
try(f())  # Error!
}
\seealso{
\code{\link[=arg_supplied]{arg_supplied()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_equal.R
\name{arg_equal}
\alias{arg_equal}
\alias{arg_not_equal}
\title{Check Equal Arguments}
\usage{
arg_equal(
  x,
  x2,
  ...,
  .arg = rlang::caller_arg(x),
  .arg2 = rlang::caller_arg(x2),
  .msg = NULL,
  .call
)

arg_not_equal(
  x,
  x2,
  ...,
  .arg = rlang::caller_arg(x),
  .arg2 = rlang::caller_arg(x2),
  .msg = NULL,
  .call
)
}
\arguments{
\item{x, x2}{the objects to compare with each other.}

\item{\dots}{other arguments passed to \code{\link[=all.equal]{all.equal()}}.}

\item{.arg, .arg2}{the names of the objects being compared; used in the error message if triggered. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether two arguments are equal (\code{arg_equal()}) or not equal (\code{arg_not_equal()}).
}
\details{
Tests for equality are performed by evaluating whether \code{isTRUE(all.equal(x, x2, ...))} is \code{TRUE} or \code{FALSE}.
}
\examples{
f <- function(x, y, ...) {
  arg_equal(x, y, ...)
}

try(f(x = 1, y = 1))      ## No error
try(f(x = 1L, y = 1.0))   ## No error despite type difference
try(f(x = 1, y = 1.00001, ## No error, within tolerance
      tolerance = .001))

try(f(x = 1, y = 2))       ## Error: different
try(f(x = 1, y = 1.00001)) ## Error

g <- function(x, y, ...) {
  arg_not_equal(x, y, ...)
}

try(g(x = 1, y = 1)) ## Error
try(g(x = 1, y = 2)) ## No error
}
\seealso{
\code{\link[=all.equal]{all.equal()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_unique.R
\name{arg_unique}
\alias{arg_unique}
\title{Check Unique Argument}
\usage{
arg_unique(x, ..., .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{\dots}{further argument passed to \code{\link[=anyDuplicated]{anyDuplicated()}}, which performs the checking.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument contains only unique values.
}
\examples{
f <- function(z) {
  arg_unique(z)
}

try(f(1:3))     # No error
try(f(NULL))    # No error for NULL
try(f(c(1, 1))) # Error: repeated values
}
\seealso{
\code{\link[=anyDuplicated]{anyDuplicated()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg-package.R
\docType{package}
\name{arg-package}
\alias{arg}
\alias{arg-package}
\title{arg: Clean and Simple Argument Checking}
\description{
Checks function arguments, ideally for use in R packages. Uses a simple interface and produces clean, informative error messages using 'cli'.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ngreifer.github.io/arg/}
  \item \url{https://github.com/ngreifer/arg}
  \item Report bugs at \url{https://github.com/ngreifer/arg/issues}
}

}
\author{
\strong{Maintainer}: Noah Greifer \email{noah.greifer@gmail.com} (\href{https://orcid.org/0000-0003-3067-7154}{ORCID})

Authors:
\itemize{
  \item Noah Greifer \email{noah.greifer@gmail.com} (\href{https://orcid.org/0000-0003-3067-7154}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_no_NA.R
\name{arg_no_NA}
\alias{arg_no_NA}
\alias{arg_is_NA}
\alias{arg_all_NA}
\title{Check NA in Argument}
\usage{
arg_no_NA(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_is_NA(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_all_NA(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument does not contain any \code{NA} values (\code{arg_no_NA()}), contains only \code{NA} values (\code{arg_all_NA()}), or is a scalar \code{NA} (\code{arg_is_NA()}).
}
\details{
\code{arg_no_NA()} throws an error when \code{anyNA(x)} is \code{0}. \code{arg_all_NA()} throws an error when \code{all(is.na(x))} is not \code{FALSE}. \code{arg_is_NA()} throws an error when \code{length(x)} is not 1 or \code{anyNA(x)} is \code{FALSE}.

\code{arg_no_NA()} is useful for checking that a meaningful argument was supplied. \code{arg_all_NA()} and \code{arg_is_NA()} are primarily used for in \code{\link[=arg_or]{arg_or()}} to denote that \code{NA} is an allowed argument.
}
\examples{
f <- function(x) {
  arg_no_NA(x)  ## x must not be NA
}

try(f(1))           ## No error
try(f(NA))          ## Error: x is NA
try(f(c(1, NA, 3))) ## Error: x contains NA

f2 <- function(y) {
  arg_all_NA(y) ## y must be NA
}

try(f2(NA))          ## No error
try(f2(c(NA, NA)))   ## No error
try(f2(1))           ## Error: y is not NA
try(f2(c(1, NA, 3))) ## Error: y is not all NA
}
\seealso{
\code{\link[=arg_non_null]{arg_non_null()}}, \code{\link[=arg_supplied]{arg_supplied()}}, \code{\link[=anyNA]{anyNA()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_element.R
\name{arg_element}
\alias{arg_element}
\alias{arg_not_element}
\title{Check Element}
\usage{
arg_element(x, values, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_not_element(x, values, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{values}{the values to check \code{x} against.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether values in an argument are all elements of some set (\code{arg_element()}) or are not all elements of a set (\code{arg_not_element()}). \code{arg_element()} throws an error when \code{all(is.element(x, values))} is \code{FALSE}, and \code{arg_not_element()} throws an error when \code{any(is.element(x, values))} is \code{TRUE}.
}
\details{
\code{arg_element()} can be useful for checking whether an argument matches one or more allowed values. It's important that a check is done beforehand to ensure \code{x} is the correct type, e.g., using \code{\link[=arg_string]{arg_string()}} or \code{\link[=arg_character]{arg_character()}} if \code{values} is a character vector. No partial matching is used; the values must match exactly. Use \code{\link[=match_arg]{match_arg()}} to allow partial matching and return the full version of the supplied argument.

\code{arg_not_element()} can be useful for ensuring there is no duplication of values between an argument and some other set.
}
\examples{
f <- function(z) {
  arg_element(z, c("opt1", "opt2", "opt3"))
}

try(f("opt1"))            # No error
try(f(c("opt1", "opt2"))) # No error, all are elements
try(f(c("opt1", "opt1"))) # No error: repeats allowed
try(f("bad_arg"))         # Error: not an element of set
try(f("opt"))             # Error: partial matching not allowed
try(f(c("opt1", "bad_arg"))) # Error: one non-match

g <- function(z) {
  arg_not_element(z, c("bad1", "bad2", "bad3"))
}

try(g("bad1"))            # Error: z is an element
try(g(c("bad1", "opt2"))) # Error: at least one bad match
try(g("opt1"))            # No error: not an element
try(g(c("opt1", "opt2"))) # No error, none are elements
}
\seealso{
\code{\link[=is.element]{is.element()}}, \code{\link[=match_arg]{match_arg()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/when_supplied.R
\name{when_supplied}
\alias{when_supplied}
\alias{when_not_null}
\title{Check Arguments When Supplied}
\usage{
when_supplied(x, ..., .arg = rlang::caller_arg(x), .call)

when_not_null(x, ..., .arg = rlang::caller_arg(x), .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{\dots}{\verb{arg_*()} functions or unevaluated function calls to be applied to \code{x}. See Examples.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
These functions check arguments only when they are supplied (\code{when_supplied()}) or when not \code{NULL} (\code{when_not_null()}). Multiple checks can be applied in sequence. This allows arguments not to have to be supplied, but checks them only if they are.
}
\details{
An error will be thrown only if \code{x} is supplied and fails one of the supplied checks (\code{when_supplied()}) or is not \code{NULL} and fails one of the supplied checks (\code{when_not_null()}).

The \code{...} arguments can be passed either as functions, e.g.,

\if{html}{\out{<div class="sourceCode">}}\preformatted{when_supplied(x,
              arg_number,
              arg_gt)
}\if{html}{\out{</div>}}

or as unevaluated function calls with the \code{x} argument absent, e.g.,

\if{html}{\out{<div class="sourceCode">}}\preformatted{when_supplied(x,
              arg_number(),
              arg_gt(bound = 0))
}\if{html}{\out{</div>}}

or as a mixture of both.

\code{when_supplied()} only makes sense to use for an argument that has no default value but which can be omitted. \code{when_not_null()} makes sense to use for an argument with a default value of \code{NULL}.
}
\examples{
f <- function(z) {
  when_supplied(z,
                arg_number,
                arg_between(c(0, 1)))
}

try(f())    # No error: not supplied
try(f("a")) # Error: not a number
try(f(2))   # Error: not within 0-1 range
try(f(.7))  # No error: number within range

g <- function(z = NULL) {
  when_not_null(z,
                arg_number,
                arg_between(c(0, 1)))
}

try(g())     # No error: NULL okay
try(g(NULL)) # No error: NULL okay
try(g("a"))  # Error: not a number
try(g(2))    # Error: not within 0-1 range
try(g(.7))   # No error: number within range
}
\seealso{
\code{\link[=arg_or]{arg_or()}}, \code{\link[=arg_supplied]{arg_supplied()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_or.R
\name{arg_or}
\alias{arg_or}
\alias{arg_and}
\title{Check That Arguments Meet Some or All Criteria}
\usage{
arg_or(x, ..., .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_and(x, ..., .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{...}{\verb{arg_*()} functions or unevaluated function calls to be applied to \code{x}. See Examples.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
\code{arg_or()} checks whether an argument meets at least one given criterion. \code{arg_and()} checks whether an argument meets all given criteria.
}
\details{
For \code{arg_or()}, an error will be thrown only if \code{x} fails all of the supplied checks. This can be useful when an argument can take on one of several input types. For \code{arg_and()}, and error will be thrown if \code{x} fails any of the supplied checks. This is less useful on its own, as the checks can simply occur in sequence without \code{arg_and()}, but \code{arg_and()} can be supplied to the \code{...} argument of \code{arg_or()} to create more complicated criteria.

The \code{...} arguments can be passed either as functions, e.g.,

\if{html}{\out{<div class="sourceCode">}}\preformatted{arg_or(x,
       arg_number,
       arg_string,
       arg_flag)
}\if{html}{\out{</div>}}

or as unevaluated function calls with the \code{x} argument absent, e.g.,

\if{html}{\out{<div class="sourceCode">}}\preformatted{arg_or(x,
       arg_number(),
       arg_string(),
       arg_flag())
}\if{html}{\out{</div>}}

or as a mixture of both.

These functions do their best to provide a clean error message composed of all the error messages for the failed checks. With many options, this can yield a complicated error message, so use caution. \code{arg_and()} marks with a check (\code{v}) any passed checks and with a cross (\code{x}) any failed checks.
}
\examples{
# `arg_or()`
f <- function(z) {
  arg_or(z,
         arg_number,
         arg_string,
         arg_flag)
}

try(f(1))      # No error
try(f("test")) # No error
try(f(TRUE))   # No error
try(f(1:4))    # Error: neither a number, string,
#              #        or flag, but a vector

# `arg_and()`
g <- function(z) {
  arg_and(z,
          arg_counts,
          arg_length(len = 2),
          arg_lt(bound = 5))
}

try(g(c(1, 2)))     # No error
try(g(c(1, 7)))     # Error: not < 5
try(g(c(1.1, 2.1))) # Error: not counts
try(g(4))           # Error: not length 2
try(g("bad"))       # Error: no criteria satisfied

# Chaining together `arg_and()` and `arg_or()`
h <- function(z) {
  arg_or(z,
         arg_all_NA,
         arg_and(arg_count,
                 arg_lt(5)),
         arg_and(arg_string,
                 arg_element(c("a", "b", "c"))))
}

try(h(NA))  # No error
try(h(1))   # No error
try(h("a")) # No error
try(h(7))   # Error: not < 5
try(h("d")) # Error: not in "a", "b", or "c"
}
\seealso{
\code{\link[=arg_supplied]{arg_supplied()}}, \code{\link[=when_not_null]{when_not_null()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_non_null.R
\name{arg_non_null}
\alias{arg_non_null}
\alias{arg_null}
\title{Check NULL Argument}
\usage{
arg_non_null(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_null(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument is non-\code{NULL} (\code{arg_non_null()}) or is \code{NULL} (\code{arg_null()}). \code{arg_non_null()} throws an error when \code{length(x)} is \code{0}. \code{arg_null()} throws an error when \code{length(x)} is not \code{0}.
}
\details{
Here, \code{NULL} refers to any length-0 object, including \code{NULL}, \code{logical(0L)}, \code{list()}, \code{3[FALSE]}, etc. \code{arg_non_null()} is useful for checking that a meaningful argument was supplied. \code{arg_null()} is primarily used for in \code{\link[=arg_or]{arg_or()}} to denote that \code{NULL} is an allowed argument.
}
\examples{
f <- function(x = NULL, y = NULL) {
  arg_non_null(x) ## x must not be NULL
  arg_null(y)     ## y must be NULL
}

try(f(x = 1,    y = NULL)) ## No error
try(f(x = NULL, y = NULL)) ## Error: x is NULL
try(f(x = 1,    y = 1))    ## Error: y is non-NULL

# Any object of length 0 is considered NULL
try(f(x = numeric())) ## Error: x is NULL
try(f(x = list()))    ## Error: x is NULL

test <- c(1, 2)[c(FALSE, FALSE)]
try(f(x = test))      ## Error: x is NULL

# arg_null() is best used in and_or():
f2 <- function(z) {
  arg_or(z,
         arg_null(),
         arg_number())
}

try(f2(NULL)) ## No error; z can be NULL
try(f2(1))    ## No error; z can be a number
try(f2(TRUE)) ## Error: z must be NULL or a number
}
\seealso{
\code{\link[=arg_length]{arg_length()}}, \code{\link[=arg_no_NA]{arg_no_NA()}}, \code{\link[=arg_supplied]{arg_supplied()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_named.R
\name{arg_named}
\alias{arg_named}
\alias{arg_colnamed}
\title{Check Named Argument}
\usage{
arg_named(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)

arg_colnamed(x, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument has valid (non-\code{NULL}, non-empty, and non-\code{NA}) names.
}
\details{
\code{arg_named()} works for vectors, lists, and data frames. To check whether a matrix has column names, use \code{arg_colnamed()} (which also works for data frames, but not vectors or lists).
}
\examples{
obj <- c(1,
         B = 2,
         C = 3)

try(arg_named(obj)) # Error: one name is blank

names(obj)[1L] <- "A"
try(arg_named(obj)) # No error

obj2 <- unname(obj)
try(arg_named(obj2)) # Error: no names

# Matrix and data frame
mat <- matrix(1:6, ncol = 2L)
colnames(mat) <- c("A", "B")

try(arg_named(mat))    # Error: matrices are not named
try(arg_colnamed(mat)) # No error

dat <- as.data.frame(mat)
try(arg_named(dat))    # No error: data frames are named
try(arg_colnamed(dat)) # No error

colnames(mat) <- NULL
try(arg_colnamed(mat)) # Error: no colnames
}
\seealso{
\code{\link[rlang:is_named]{rlang::is_named()}}, \code{\link[=names]{names()}}, \code{\link[=colnames]{colnames()}}, \code{\link[=arg_data]{arg_data()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arg_length.R
\name{arg_length}
\alias{arg_length}
\title{Check Argument Length}
\usage{
arg_length(x, len = 1L, .arg = rlang::caller_arg(x), .msg = NULL, .call)
}
\arguments{
\item{x}{the argument to be checked}

\item{len}{\code{integer}; the allowed length(s) of \code{x}. Default is 1.}

\item{.arg}{the name of the argument supplied to \code{x} to appear in error messages. The default is to extract the argument's name using \code{\link[rlang:caller_arg]{rlang::caller_arg()}}. Ignored if \code{.msg} is supplied.}

\item{.msg}{an optional alternative message to display if an error is thrown instead of the default message.}

\item{.call}{the execution environment of a currently running function, e.g. \code{.call = rlang::current_env()}. The corresponding function call is retrieved and mentioned in error messages as the source of the error. Passed to \code{\link[=err]{err()}}. Set to \code{NULL} to omit call information. The default is to search along the call stack for the first user-facing function in another package, if any.}
}
\value{
Returns \code{NULL} invisibly if an error is not thrown.
}
\description{
Checks whether an argument has a specified length.
}
\details{
\code{len} can contain multiple allowed counts; an error will be thrown only if \code{length(x)} is not equal to any value of \code{len}.
}
\examples{
obj <- 1:4

try(arg_length(obj, 1))
try(arg_length(obj, 4))
try(arg_length(obj, c(1, 4)))

# These test the same thing:
try(arg_length(obj, c(0:3)))
try(when_not_null(obj,
                  arg_length(1:3)))
}
\seealso{
\code{\link[=length]{length()}}, \code{\link[=arg_non_null]{arg_non_null()}} to specifically test that an object's length is or is not 0.
}
